// pGPSmap, Data preparing utility for cGPSmapper
// Version 1.0 beta 9
// (C) 2002 by Olexa Riznyk (www.olexa.com.ua)
// 
// This is not the original file!
// changed splitting option from 5x5 to 2x2
// added generating batchfile for cGPSmapper
//
// Is it necessary to rebiuld maps built by previous version?
//
// 1.0 beta 4: If "Cannot merge polygon" ErrorMessage was generated, it is recommended to rebiuld the map, because some data may be lost
// All earlier versions: Rebuilding may make the map a bit smaller and faster
//
// Features added in 1.0 beta 9:
//
// 1. Now /names option also accepts base block names for mapping
//
// Bugs fixed in 1.0 beta 9:
//
// 1. Blocks sticking disabled as it is not finished yet
//
// Features added in 1.0 beta 8:
//
// 1. Now DataN instead of OriginN in [RGN10] and [RGN20] is also supported
// 2. SK-42 datum is automatically converted to WGS84 (W84)
// 3. Splitting polylines, splitting regions and merging holes made optional
// 4. Timing summary added
// 5. Added option for loading submap's names from file
// 6. Added option for skipping blocks
// 7. Added option for sticking blocks to other blocks
//
// Bugs fixed in 1.0 beta 7:
//
// 1. Splitting crashed if there were some lines between blocks
//
// Features added in 1.0 beta 6:
//
// 1. Map splitting mode added
//
// Features added in 1.0 beta 5:
//
// 1. Exploding algorythm speeded up
// 2. StatusMessage, WarningMessage and ErrorMessage messages made more informative
// 3. Fake nodes in source polygons are now deleted during import
// 4. Fake nodes make not not appearing during merging and splitting (this makes the algorythm slower)
// 5. Merging at one point if inner and outer polygons have common node made possible
// 6. Polylines in [RGN40] sections are also now accepted and processed (for compatibility with cGPSmap 00412)
//
// Bugs fixed in 1.0 beta 5:
//
// 1. Corrected duplicated polygons processing
// 2. Corrected source object line number displayed in StatusMessage (object last line number was displayed)
//
// Features added in 1.0 beta 4:
//
// 1. Regions are split also if they cannot fit one TRE for specific layer
// 2. Coordinates are always output with 6 digits after decimal separator, no trailing zeroes supression
// 3. Maximum number of nodes reduced to 249 to workaround cGPSmap 00442's buggy polygon splitting algorythm
// 4. If some inner polygons cannot be merged with outer one, they are deleted and ErrorMessage message is generated
// 5. Windows Script Host version check added (5.5 is required)
//
// Features added in 1.0 beta 3:
//
// 1. Coordinates are rounded to 8 digits after decimal separator
//
// Bugs fixed in 1.0 beta 3:
//
// 1. In some cases inner polygons were not merged with outer ones
//
// Bugs fixed in 1.0 beta 2:
//
// 1. Common elements were exported for first polygon only
//
// To do:
// 
// 1.	Add clipping by region mode; ready for points and polylines, make for regions
// 2.	Make status detalization speed-dependent
// 3.	Add options for filtering by object class: POI, city, polyline, region
// 4.	Generate more readable names for map blocks (by cities' names)
// 5.	Correct map border processing algorythm (if necessary)
// 6.	In splitting mode detect whether source map is already splitting result and calculate base ID correctly.
// 7.	Make line splitting algorythm one-pass
// 8.	Check polygons' borders for self-crossing and fix if necessary (make it optional)
// 9.	Check polygons within region for crossing each other and fix if necessary (make it optional)
// 10.	Provide generalization of regions and polylines, including merging regions with polylines that represent the same object

var sVersion = "1.0 beta 9";
var sSplash = "pGPSmap, Data preparing utility for cGPSmapper\n"
	+ "Version " + sVersion + "\n"
	+ "(C) 2002 by Olexa Riznyk (www.olexa.com.ua)\n"
  + "\n"
  + "\n"
  + "This is not the original file!\n"
  + "changed splitting option 1 from 5x5 to 2x2\n"
  + "added generating batchfile for cGPSmapper\n";
  var sHelp = sSplash
	+ "\n"
	+ "Usage: pGPSmap [options] <source_file> {<destination_file> | [<destination_folder>]}\n"
	+ "\n"
	+ "<source_file>		Source file name\n"
	+ "<destination_file>	Destination file name (non-splitting mode)\n"
	+ "<destination_folder>	Destination folder name, optional (splitting mode)\n"
	+ "\n"
	+ "Options:\n"
	+ "\n"
	+ "/split=<N> Split map into submaps. Valid values for <N>:\n"
	+ "           0 - do not split map (default)\n"
	+ "           1 - 2 by latitude, 2 by longitude\n"
	+ "           2 - 4 by latitude, 6 by longitude, 2x and 4x near poles\n"
	+ "           3 - 2 by latitude, 3 by longitude, 2x and 4x near poles\n"
	+ "           4 - 1 by latitude, 040' by longitude, 2x and 4x near poles\n"
	+ "           5 - 030' by latitude, 020' by longitude, 2x and 4x near poles\n"
	+ "\n"
	+ "/names=<names_file_name>\n"
	+ "\n"
	+ "           Names for submaps; tab-separated text file with block IDs or block\n"
	+ "           base names in first column and block names in second; only effective\n"
	+ "           with /split option\n"
	+ "\n"
	+ "/skipblocks=<block>[,...]\n"
	+ "\n"
	+ "           Skip some blocks during splitting; <block> can be either block names\n"
	+ "           or block IDs in either decimal or hexadecimal format; only effective\n"
	+ "           with /split option\n"
	+ "\n"
	//+ "/stickblocks=[<target>,<source>[,<source>[...]][;...]\n"
	//+ "\n"
	//+ "           Stick some blocks to their neighbouring blocks; <target> and\n"
	//+ "           <source> can be either block names or block IDs in either decimal or\n"
	//+ "           hexadecimal format\n"
	//+ "\n"
	//+ "/cut=<cut_file_name>\n"
	//+ "\n"
	//+ "           Cut map by the region contained in <cut_file_name> file;\n"
	//+ "           each layer is cut by corresponding layer in cutting region;\n"
	//+ "           <cut_file_name> file may contain several [RGN80] sections,\n"
	//+ "           but all of them will be treated as one region\n"
	//+ "\n"
	+ "/splitpolylines={y|n}\n"
	+ "\n"
	+ "           Whether to split polylines which exceed maximum number of nodes or\n"
	+ "           maximum TRE size; default is no\n"
	+ "\n"
	+ "/splitregions={y|n}\n"
	+ "\n"
	+ "           Whether to split regions which exceed maximum number of nodes or\n"
	+ "           maximum TRE size; default is no\n"
	+ "\n"
	+ "/mergeholes={y|n}\n"
	+ "\n"
	+ "           Whether to merge holes with regions' borders (convert regions to\n"
	+ "           polygons); default is yes\n"
	;
//--------------------------------
var MaxNodesCnt = 249;
var CoordinatesPrecision = 6;
var o = 0.000000000001;
//--------------------------------
var DestinationFile, BlockFiles, PendingSections;
var SectionsProcessed, NodesProcessed, POIsProcessed, CitiesPointsProcessed, PolylinesProcessed, LinesCreated, RegionsProcessed, PolygonsCreated;
var CurrentStatus = "";
//--------------------------------
var FileHeader = new Array();
var MapSection, ImgSection;
var MapParams = new Array();
var ImgParams = new Array();
var LevelGrid = new Array();
var BlockIdFromatIsHex, BaseBlockID;
//--------------------------------
var fso = new ActiveXObject("Scripting.FileSystemObject");
var ForReading = 1, ForWriting = 2, ForAppending = 8;
var TristateTrue = -1, TristateFalse = 0, TristateUseDefault = -2;
//-------------------------------------------
function LTrim(Str) {
	var Chr;
	while (Chr = Str.substr(0, 1), Chr == " " || Chr == "\t" || Chr == "\r" || Chr == "\n")
		Str = Str.substr(1);
	return Str;
}
//-------------------------------------------
function RTrim(Str) {
	var Chr;
	while (Chr = Str.substr(Str.length - 1, 1), Chr == " " || Chr == "\t" || Chr == "\r" || Chr == "\n")
		Str = Str.substr(0, Str.length - 1);
	return Str;
}
//-------------------------------------------
function Trim(Str) {
	return LTrim(RTrim(Str));
}
//-------------------------------------------
function FillLeft(Str, Len, Fill) {
	while (String(Str).length < Len)
		Str = (String(Fill) != "undefined" ? Fill : " ") + Str;
	return Str;
}
//-------------------------------------------
function Roman(Number) {
	return "XXX".substr(0, (Number - Number % 10) / 10)
	+ (Number % 10 == 9 ? "IX"
	: Number % 10 >= 5 ? "V" + "III".substr(0, Number % 10 - 5)
	: Number % 10 == 4 ? "IV"
	: "III".substr(0, Number % 10));
}
//-------------------------------------------
var ro = 206264.8062; // Number of angular seconds in radian

// Krasovsky ellipsoid (SK-42)
var aP = 6378245; // Semi-major axis
var alP = 1 / 298.3; // Inverse flattening
var e2P = 2 * alP - Math.pow(alP, 2); // Suqare of eccentricity

// GRS80 ellipsoid (WGS84)
var aW = 6378137; // Semi-major axis
var alW = 1 / 298.257223563; // Inverse flattening
var e2W = 2 * alW - Math.pow(alW, 2); // Suqare of eccentricity

// Auxiliary values for ellipsoids transformation
var a = (aP + aW) / 2;
var e2 = (e2P + e2W) / 2;
var da = aW - aP;
var de2 = e2W - e2P;

// Linear transformation elements, in meters
var dx = 23.92;
var dy = -141.27;
var dz = -80.9;
// Angular transformation elements, in meters
var wx = 0;
var wy = -0.35;
var wz = -0.82;
// Differential scale distinction
var ms = -0.12 * Math.pow(10, -6);
//-------------------------------------------
function WGS84Lat(Bd, Ld, H) {
  var B, L, M, N, dB;
  
  B = Bd * Math.PI / 180;
  L = Ld * Math.PI / 180;
  M = a * (1 - e2) / Math.pow(1 - e2 * Math.pow(Math.sin(B), 2), 1.5);
  N = a * Math.pow(1 - e2 * Math.pow(Math.sin(B), 2), -0.5);
  
  dB = ro / (M + H) * (N / a * e2 * Math.sin(B) * Math.cos(B) * da
  + (Math.pow(N, 2) / Math.pow(a, 2) + 1) * N * Math.sin(B) * Math.cos(B) * de2 / 2
  - (dx * Math.cos(L) + dy * Math.sin(L)) * Math.sin(B) + dz * Math.cos(B))
  - wx * Math.sin(L) * (1 + e2 * Math.cos(2 * B))
  + wy * Math.cos(L) * (1 + e2 * Math.cos(2 * B))
  - ro * ms * e2 * Math.sin(B) * Math.cos(B);
  
  return Bd + dB / 3600;
}
//-------------------------------------------
function WGS84Long(Bd, Ld, H) {
  var B, L, N, dL;
  
  B = Bd * Math.PI / 180;
  L = Ld * Math.PI / 180;
  N = a * Math.pow(1 - e2 * Math.pow(Math.sin(B), 2), -0.5);
  
  dL = ro / ((N + H) * Math.cos(B)) * (-dx * Math.sin(L) + dy * Math.cos(L))
  + Math.tan(B) * (1 - e2) * (wx * Math.cos(L) + wy * Math.sin(L)) - wz;
  
  return Ld + dL / 3600;
}
//-------------------------------------------
function WGS84Alt(Bd, Ld, H) {
  var B, L, N, dH
  
  B = Bd * Math.PI / 180;
  L = Ld * Math.PI / 180;
  N = a * Math.pow(1 - e2 * Math.pow(Math.sin(B), 2), -0.5);
  
  dH = -a / N * da + N * Math.pow(Math.sin(B), 2) * de2 / 2
  + (dx * Math.cos(L) + dy * Math.sin(L)) * Math.cos(B) + dz * Math.sin(B)
  - N * e2 * Math.sin(B) * Math.cos(B) * (wx / ro * Math.sin(L) - wy / ro * Math.cos(L))
  + (Math.pow(a, 2) / N + H) * ms;
  
  return H + dH;
}
//-------------------------------------------
function StatusMessage(NewStatus) {
	var Padding, CharNum;
	
	if (!CommandLineMode())
		return;
		
	for (Padding = "", CharNum = 0; CharNum < CurrentStatus.length - NewStatus.length; CharNum++, Padding += " ");

	WScript.StdOut.Write("\r" + NewStatus + Padding);
	CurrentStatus = NewStatus;
}
//-------------------------------------------
function WarningMessage(Message) {
	var Padding, CharNum;
	
	if (!CommandLineMode())
		return;
		
	for (Padding = "", CharNum = 0; CharNum < CurrentStatus.length - Message.length; CharNum++, Padding += " ");

	WScript.StdOut.Write("\r" + Message + Padding + "\n");
	CurrentStatus = "";
}
//-------------------------------------------
function ErrorMessage(Message) {
	var Padding, CharNum;
	
	if (!CommandLineMode()) {
		WScript.Echo(Message);
	} else {
		for (Padding = "", CharNum = 0; CharNum < CurrentStatus.length - Message.length; CharNum++, Padding += " ");

		WScript.StdOut.Write("\r" + Message + Padding + "\n");
		CurrentStatus = "";
	}
}
//-------------------------------------------
function ArrayIndexOf(Array, Value) {
	var Index;
	
	for (Index in Array)
		if (Array[Index] == Value)
			return Index;

	return -1;
}
//-------------------------------------------
function PointWithinLink(Point, Point1, Point2) {

	if (Point1.Latitude == Point2.Latitude)
		return Point.Latitude == Point1.Latitude
		&& (Point.Longitude >= Point1.Longitude && Point.Longitude <= Point2.Longitude
		|| Point.Longitude <= Point1.Longitude && Point.Longitude >= Point2.Longitude);
	else
		return (Point.Latitude - Point2.Latitude) * (Point1.Longitude - Point2.Longitude)
		/ (Point1.Latitude - Point2.Latitude) == Point.Longitude - Point2.Longitude;
}
//-------------------------------------------
function PointWithinPolygon(Point, Polygon) {
	// Tests point for being entirely within polygon
	// Returns 1 if point is entirely within polygon, 0 if it is on its border and -1 if it is outside polygon
	var Result, LastTouchingLinkSide, FirstNodeNum, FirstNodePassed, NodeNum, PrevNode, Node, NextNodeNum;
	var Longitude, Latitude0, Longitude0, Latitude1, Longitude1, Latitude2, Longitude2;

	if (Point.Latitude < Polygon.MinLatitude
	|| Point.Latitude > Polygon.MaxLatitude
	|| Point.Longitude < Polygon.MinLongitude
	|| Point.Longitude > Polygon.MaxLongitude)
		return -1;

	Result = false;
	LastTouchingLinkSide = 0;
	FirstNodeNum = 0;
	FirstNodePassed = false;
	PrevNode = Polygon.Nodes[0];
	for (NodeNum = 1; NodeNum != FirstNodeNum + 1 || !FirstNodePassed; NodeNum = NodeNum == Polygon.Nodes.length - 1 ? 0 : NodeNum + 1, PrevNode = Node) {
		Node = Polygon.Nodes[NodeNum];
		if (PrevNode.Latitude < Point.Latitude && Node.Latitude > Point.Latitude
		|| PrevNode.Latitude > Point.Latitude && Node.Latitude < Point.Latitude) {
			// Link crosses point's latitude
			Longitude = PrevNode.Longitude
			+ (Node.Longitude - PrevNode.Longitude) * (Point.Latitude - PrevNode.Latitude) / (Node.Latitude - PrevNode.Latitude);
			if (Longitude == Point.Longitude)
				// Link crosses the point
				return 0;
			else if (Longitude < Point.Longitude)
				// Link passes under the point
				Result = !Result;
			FirstNodePassed = true;
		} else if (PrevNode.Latitude == Point.Latitude && Node.Latitude == Point.Latitude) {
			// Link is entirely within point's latitude
			if (PrevNode.Longitude <= Point.Longitude && Node.Longitude >= Point.Longitude
			|| PrevNode.Longitude >= Point.Longitude && Node.Longitude <= Point.Longitude)
				// Link crosses the point
				return 0;
			if (FirstNodeNum == NodeNum - 1 && Node.Longitude < Point.Longitude)
				// There was no any link that crosses point's latitude or finishes at it yet
				FirstNodeNum++;
		} else if (PrevNode.Latitude != Point.Latitude && Node.Latitude == Point.Latitude) {
			// Link finishes at point's latitude
			if (Node.Longitude == Point.Longitude)
				// Link crosses the point
				return 0;
			else if (Node.Longitude < Point.Longitude)
				// Remember last touching link side
				LastTouchingLinkSide = PrevNode.Latitude < Point.Latitude ? -1 : 1;
			FirstNodePassed = true;
		} else if (PrevNode.Latitude == Point.Latitude && Node.Latitude != Point.Latitude) {
			// Link starts at point's latitude
			if (PrevNode.Longitude == Point.Longitude)
				// Link crosses the point
				return 0;
			else if (PrevNode.Longitude < Point.Longitude)
				if (LastTouchingLinkSide == 0)
					// There was no touching link yet. We should loop through the polygon 'till this node
					FirstNodeNum = NodeNum;
				else if (LastTouchingLinkSide == -1 && Node.Latitude > Point.Latitude
				|| LastTouchingLinkSide == 1 && Node.Latitude < Point.Latitude)
					// This liks with previous touching link together cross point's latitude
					Result = !Result;
		} else
			// Link does not cross point's latitude
			FirstNodePassed = true;
	}
	return Result ? 1 : -1;
}
//-------------------------------------------
function PointWithinRegion(Point, Region) {
	// Tests point for being within region
	// Returns 1 if point is entirely within region, 0 if it is on its border and -1 if it is outside polygon
	var IsWithin, PolygonID;

	IsWithin = -1;
	for (PolygonID in Region) {
		switch (PointWithinPolygon(Point, Region[PolygonID])) {
		case 0:
			return 0;
		case 1:
			IsWithin = -IsWithin;
			break;
		}
	}
	
	return IsWithin;
}
//-------------------------------------------
function GetAnyInternalPoint(Polygon) {
	// Returns any internal point of polygon
	var PrevNode, Node, NextNode, NodeNum, Point, DistanceReduced;
	
	if (Polygon.Area > 0)
		// Loop through all nodes as vertexes of angles
		for (NodeNum = 0,
		PrevNode = Polygon.Nodes[Polygon.Nodes.length - 2],
		Node = Polygon.Nodes[Polygon.Nodes.length - 1],
		NextNode = Polygon.Nodes[0];
		NodeNum < Polygon.Nodes.length;
		NodeNum++, PrevNode = Node, Node = NextNode, NextNode = Polygon.Nodes[NodeNum]) {
		
			// Check node for being fake node
			if (PrevNode.Latitude == Node.Latitude && PrevNode.Longitude == Node.Longitude
			|| NextNode.Latitude == Node.Latitude && NextNode.Longitude == Node.Longitude
			|| PointWithinLink(Node, PrevNode, NextNode))
				continue;
		
			// Find internal point for current angle
			Point = new Object();
			for (Point.Latitude = (PrevNode.Latitude + NextNode.Latitude) / 2,
			Point.Longitude = (PrevNode.Longitude + NextNode.Longitude) / 2,
			DistanceReduced = true;
			Point.Latitude != Node.Latitude || Point.Longitude != Node.Longitude;
			Point.Latitude = DistanceReduced ? 2 * Node.Latitude - Point.Latitude : (Point.Latitude + Node.Latitude) / 2,
			Point.Longitude = DistanceReduced ? 2 * Node.Longitude - Point.Longitude : (Point.Longitude + Node.Longitude) / 2,
			DistanceReduced = !DistanceReduced)
			
				if (PointWithinPolygon(Point, Polygon) == 1)
					return Point;
		}
}
//-------------------------------------------
function PolygonWithinPolygon(InnerPolygon, OuterPolygon) {
	// Test first polygon for being entirely within second polygon
	// Assumed that either one polygon is entirely within other polygon or polygons intersection is not a polygon
	// Returns -1 if InnerPolygon is not entirely within OuterPolygon
	// , 1 if InnerPolygon is entirely within OuterPolygon
	// or 0 if polygons are equal
	var NodeID, PolygonsAreEqual;
	var PrevInnerNode, InnerNode, NextInnerNode, OuterNodeNum, OuterNode, PrevOuterNode;
	var InternalPoint;

	if (InnerPolygon.MinLatitude < OuterPolygon.MinLatitude
	|| InnerPolygon.MinLongitude < OuterPolygon.MinLongitude
	|| InnerPolygon.MaxLatitude > OuterPolygon.MaxLatitude
	|| InnerPolygon.MaxLongitude > OuterPolygon.MaxLongitude
	|| InnerPolygon.Area > OuterPolygon.Area)
		return -1;
		
	for (NodeID in InnerPolygon.Nodes)
		switch (PointWithinPolygon(InnerPolygon.Nodes[NodeID], OuterPolygon)) {
		case -1:
			return -1;
		case 1:
			return 1;
		}
	
	PolygonsAreEqual = true;
	for (NodeID in OuterPolygon.Nodes)
		switch (PointWithinPolygon(OuterPolygon.Nodes[NodeID], InnerPolygon)) {
		case -1:
			PolygonsAreEqual = false;
			break;
		case 1:
			return -1;
		}
	
	if (PolygonsAreEqual)
		return 0;
		
	// Now all nodes of inner polygon are on the border of outer polygon
	// We need to find whether inner polygon is within, or not within outer polygon
	
	if (InnerPolygon.Area == 0)
		return 1;

	InternalPoint = GetAnyInternalPoint(InnerPolygon);
	if (String(InternalPoint) == "undefined")
		return 1;

	return PointWithinPolygon(InternalPoint, OuterPolygon);
}
//-------------------------------------------
function cmp(arg1, arg2) {
	return arg1 == arg2 ? 0 : arg1 < arg2 ? -1 : 1;
}
//-------------------------------------------
function ExplodeRegion(Region, Regions) {
	// Divides region into an array of non-intersecting regions
	var ParentPolygonNum, PolygonIsParent, PolygonNum;
	var UsedPolygonsNums, NewRegion, ChildPolygonNum, ChildPolygonNumNum, ParentPolygonNumNum;

	while (Region.length > 0)
		for (ParentPolygonNum = 0; ParentPolygonNum < Region.length; ParentPolygonNum++) {
			// Test current polygon for not being within any other polygon
			PolygonIsParent = true;
			for (PolygonNum = 0; PolygonNum < Region.length; PolygonNum++)
				if (PolygonNum != ParentPolygonNum)
					if (PolygonWithinPolygon(Region[ParentPolygonNum], Region[PolygonNum]) == 1) {
						PolygonIsParent = false;
						break;
					}
			if (PolygonIsParent) {
				// This polygon is one of parents
				UsedPolygonsNums = new Array();
				NewRegion = new Array();
				NewRegion.push(Region[ParentPolygonNum]);

				// Find this polygon's children
				for (ChildPolygonNum = 0; ChildPolygonNum < Region.length; ChildPolygonNum++)
					if (ChildPolygonNum != ParentPolygonNum)
						if (PolygonWithinPolygon(Region[ChildPolygonNum], Region[ParentPolygonNum]) == 1)
							UsedPolygonsNums.push(ChildPolygonNum);

				// Delete numbers of children that are within some other children
				for (ChildPolygonNumNum = UsedPolygonsNums.length - 1; ChildPolygonNumNum >= 0; ChildPolygonNumNum--)
					for (ParentPolygonNumNum = 0; ParentPolygonNumNum < UsedPolygonsNums.length; ParentPolygonNumNum++)
						if (ChildPolygonNumNum != ParentPolygonNumNum
						&& PolygonWithinPolygon(Region[UsedPolygonsNums[ChildPolygonNumNum]], Region[UsedPolygonsNums[ParentPolygonNumNum]]) >= 0) {
							UsedPolygonsNums.splice(ChildPolygonNumNum, 1);
							break;
						}

				// Add children polygons to region
				for (ChildPolygonNumNum = 0; ChildPolygonNumNum < UsedPolygonsNums.length; ChildPolygonNumNum++)
					NewRegion.push(Region[UsedPolygonsNums[ChildPolygonNumNum]]);

				UsedPolygonsNums.push(ParentPolygonNum);
				Regions.push(NewRegion);
				// When we got complete region, we need to remove its polygons from source list (region)
				// to make "lakes on islands" be able to become parent polygons
				UsedPolygonsNums.sort(cmp);
				while (UsedPolygonsNums.length > 0)
					Region.splice(UsedPolygonsNums.pop(),1);
				break;
			}
		}

}
//-------------------------------------------
function RegionRequiresSplitting(Region, LevelID, Options) {
	// Tests region for having too much nodes, and being too large to fit one TRE or crossing submap border
	var SplitDirection, MinLatitude, MaxLatitude, MinLongitude, MaxLongitude;
	var NextLatitude, NextLongitude, NodesCnt, PolygonNum;
	
	SplitDirection = new Object();
	SplitDirection.SplitBy = "None";
	
	MinLatitude = Region[0].MinLatitude;
	MaxLatitude = Region[0].MaxLatitude;
	MinLongitude = Region[0].MinLongitude;
	MaxLongitude = Region[0].MaxLongitude;
	
	if (Options.SplitMode == 1) {
		NextLatitude = Math.floor(MinLatitude / 5 + o) * 5 + 5;
		if (MaxLatitude > NextLatitude) {
			SplitDirection.SplitBy = "Latitude";
			SplitDirection.SplitValue = NextLatitude;
			return SplitDirection;
		}
		NextLongitude = Math.floor(MinLongitude / 5 + o) * 5 + 5;
		if (MaxLongitude > NextLongitude) {
			SplitDirection.SplitBy = "Longitude";
			SplitDirection.SplitValue = NextLongitude;
			return SplitDirection;
		}
	} else if (Options.SplitMode > 1) {
		SplitScale = Options.SplitMode == 2 ? 1 : Options.SplitMode == 3 ? 2 : Options.SplitMode == 4 ? 6 : 12;
		NextLatitude = Math.floor(MinLatitude * SplitScale / 4 + o) * 4 / SplitScale + 4 / SplitScale;
		if (MaxLatitude > NextLatitude) {
			SplitDirection.SplitBy = "Latitude";
			SplitDirection.SplitValue = NextLatitude;
			return SplitDirection;
		}
		
		if (MaxLatitude <= 60 && MinLatitude >= -60)
			NextLongitude = Math.floor((MinLongitude + 180) * SplitScale / 6 + o) * 6 / SplitScale - 180 + 6 / SplitScale;
		else if (MaxLatitude <= 76 && MinLatitude >= -76)
			NextLongitude = Math.floor((MinLongitude + 180) * SplitScale / 12 + o) * 12 / SplitScale - 180 + 12 / SplitScale;
		else
			NextLongitude = Math.floor((MinLongitude + 180) * SplitScale / 24 + o) * 24 / SplitScale - 180 + 24 / SplitScale;
			
		if (MaxLongitude > NextLongitude) {
			SplitDirection.SplitBy = "Longitude";
			SplitDirection.SplitValue = NextLongitude;
			return SplitDirection;
		}
	}
	
	if (!Options.SplitRegions)
		return SplitDirection;
		
	if (ImgParams["tresize"] != undefined && LevelGrid[LevelID] != undefined)
		if (MaxLatitude - MinLatitude > MaxLongitude - MinLongitude) {
			if (MaxLatitude - MinLatitude >= ImgParams["tresize"] * LevelGrid[LevelID]) {
				SplitDirection.SplitBy = "Latitude";
				SplitDirection.SplitValue = (MaxLatitude + MinLatitude) / 2;
				return SplitDirection;
			}
		} else if (MaxLongitude - MinLongitude >= ImgParams["tresize"] * LevelGrid[LevelID]) {
			SplitDirection.SplitBy = "Longitude";
			SplitDirection.SplitValue = (MaxLongitude + MinLongitude) / 2;
			return SplitDirection;
		}
	
	NodesCnt = (Region.length - 1) * 2; // Merging polygons adds two nodes
	for (PolygonNum in Region) {
		NodesCnt += Region[PolygonNum].Nodes.length;
		MinLatitude = Math.min(MinLatitude, Region[PolygonNum].MinLatitude);
		MaxLatitude = Math.max(MaxLatitude, Region[PolygonNum].MaxLatitude);
		MinLongitude = Math.min(MinLongitude, Region[PolygonNum].MinLongitude);
		MaxLongitude = Math.max(MaxLongitude, Region[PolygonNum].MaxLongitude);
	}
	
	if (NodesCnt > MaxNodesCnt)
		if (MaxLatitude - MinLatitude > MaxLongitude - MinLongitude) {
			SplitDirection.SplitBy = "Latitude";
			SplitDirection.SplitValue = (MaxLatitude + MinLatitude) / 2;
			return SplitDirection;
		} else {
			SplitDirection.SplitBy = "Longitude";
			SplitDirection.SplitValue = (MaxLongitude + MinLongitude) / 2;
			return SplitDirection;
		}

	return SplitDirection;
}
//-------------------------------------------
function CmpSplitChildren(Polygon1, Polygon2) {
	var FirstNode1, LastNode1, FirstNode2, LastNode2;

	FirstNode1 = Polygon1.Nodes[0];
	LastNode1 = Polygon1.Nodes[Polygon1.Nodes.length - 1];
	FirstNode2 = Polygon2.Nodes[0];
	LastNode2 = Polygon2.Nodes[Polygon2.Nodes.length - 2];
	
	if (FirstNode1.Latitude == LastNode1.Latitude)
		if (Math.min(FirstNode1.Longitude, LastNode1.Longitude) < Math.min(FirstNode2.Longitude, LastNode2.Longitude))
			return -1;
		else
			return 1;
	else
		if (Math.min(FirstNode1.Latitude, LastNode1.Latitude) < Math.min(FirstNode2.Latitude, LastNode2.Latitude))
			return -1;
		else
			return 1;

}
//-------------------------------------------
function SplitRegionTo(Region, Regions, SplitDirection) {
	// Splits regions across its longer side and pushes resulting regions into Regions array
	var SplitByLatitude, SplitLatitude, SplitLongitude, Polygons, PolygonNum, PolygonSide, SourcePolygonNum;
	var Nodes, NodeNum, FirstNodeNum, FirstNodePassed, Node, PrevNode;
	var NewNode, NewNodes, NewPolygon, FirstRegionNum, ChildrenToMerge, ChildPolygon;
	var RegionNum, CurRegion, ParentPolygon, ChildPolygonNum, ChildFirstNode, ChildLastNode;
	var LastParentNode, FirstChildNode, FirstParentNode, LastChildNode;

	// Determine whether to split region by latitude or by longitude
	SplitByLatitude = (SplitDirection.SplitBy == "Latitude");
	if (SplitByLatitude)
		SplitLatitude = SplitDirection.SplitValue;
	else
		SplitLongitude = SplitDirection.SplitValue;

	Polygons = new Array();
	for (PolygonNum = 0; PolygonNum < Region.length; PolygonNum++) {

		if ((SplitByLatitude && Region[PolygonNum].MinLatitude <= SplitLatitude && Region[PolygonNum].MaxLatitude >= SplitLatitude)
		|| (!SplitByLatitude && Region[PolygonNum].MinLongitude <= SplitLongitude && Region[PolygonNum].MaxLongitude >= SplitLongitude)) {

			// This polygon requires splitting
			PolygonSide = 0;
			Nodes = Region[PolygonNum].Nodes;
			PrevNode = Nodes[Nodes.length - 1];
			for (NodeNum = 0, FirstNodeNum = -2, FirstNodePassed = false; NodeNum != (FirstNodeNum == Nodes.length - 1 ? 0 : FirstNodeNum + 1) || !FirstNodePassed; NodeNum = NodeNum == Nodes.length - 1 ? 0 : NodeNum + 1) {
				if (NodeNum == (FirstNodeNum == Nodes.length - 1 ? 0 : FirstNodeNum + 1))
					FirstNodePassed = true;
				Node = Nodes[NodeNum];

				// Check current link for touching or crossing split line
				if ((SplitByLatitude && Node.Latitude != PrevNode.Latitude
				&& ((Node.Latitude <= SplitLatitude && PrevNode.Latitude >= SplitLatitude)
				|| (Node.Latitude >= SplitLatitude && PrevNode.Latitude <= SplitLatitude)))
				|| (!SplitByLatitude && Node.Longitude != PrevNode.Longitude
				&& ((Node.Longitude <= SplitLongitude && PrevNode.Longitude >= SplitLongitude)
				|| (Node.Longitude >= SplitLongitude && PrevNode.Longitude <= SplitLongitude)))) {
				
					// Calculate second splitting coordinate
					if (SplitByLatitude)
						SplitLongitude = PrevNode.Longitude
						+ (Node.Longitude - PrevNode.Longitude)
						* (SplitLatitude - PrevNode.Latitude) / (Node.Latitude - PrevNode.Latitude)
					else
						SplitLatitude = PrevNode.Latitude
						+ (Node.Latitude - PrevNode.Latitude)
						* (SplitLongitude - PrevNode.Longitude) / (Node.Longitude - PrevNode.Longitude)
					
					// Check current link for going to split line to finish current polygon
					if (PolygonSide != 0 &&
					((SplitByLatitude
					&& ((PolygonSide < 0 && Node.Latitude >= SplitLatitude) || (PolygonSide > 0 && Node.Latitude <= SplitLatitude)))
					|| (!SplitByLatitude
					&& ((PolygonSide < 0 && Node.Longitude >= SplitLongitude) || (PolygonSide > 0 && Node.Longitude <= SplitLongitude))))) {
						// Finish current polygon
						NewNode = new Object();
						NewNode.Latitude = SplitLatitude;
						NewNode.Longitude = SplitLongitude;
						
						NewNodes.push(NewNode);
						NewPolygon = new Object();
						NewPolygon.Nodes = NewNodes;
						SetPolygonProperties(NewPolygon);
						NewPolygon.SourcePolygons = new Array();
						for (SourcePolygonNum = 0; SourcePolygonNum < Region[PolygonNum].SourcePolygons.length; SourcePolygonNum++)
							NewPolygon.SourcePolygons.push(Region[PolygonNum].SourcePolygons[SourcePolygonNum]);
						
						Polygons.push(NewPolygon);
						PolygonSide = 0;
					}

					// Check current link for going from split line to start new polygon
					if (NodeNum != FirstNodeNum && PolygonSide == 0 &&
					((SplitByLatitude
					&& ((Node.Latitude < SplitLatitude && PrevNode.Latitude >= SplitLatitude)
					|| (Node.Latitude > SplitLatitude && PrevNode.Latitude <= SplitLatitude)))
					|| (!SplitByLatitude
					&& ((Node.Longitude < SplitLongitude && PrevNode.Longitude >= SplitLongitude)
					|| (Node.Longitude > SplitLongitude && PrevNode.Longitude <= SplitLongitude))))) {
						// Start a new polygon
						NewNodes = new Array();

						NewNode = new Object();
						NewNode.Latitude = SplitLatitude;
						NewNode.Longitude = SplitLongitude;
						NewNodes.push(NewNode);
						
						NewNode = new Object();
						NewNode.Latitude = Node.Latitude;
						NewNode.Longitude = Node.Longitude;
						NewNodes.push(NewNode);
						
						if (FirstNodeNum == -2)
							FirstNodeNum = NodeNum;
						
						if (SplitByLatitude)
							if (Node.Latitude > SplitLatitude)
								PolygonSide = 1;
							else
								PolygonSide = -1;
						else
							if (Node.Longitude > SplitLongitude)
								PolygonSide = 1;
							else
								PolygonSide = -1;
					}
				}
				
				// Check link for not touching split line to add next node to current polygon
				if (PolygonSide != 0
				&& ((SplitByLatitude
				&& (PrevNode.Latitude > SplitLatitude && Node.Latitude > SplitLatitude
				|| PrevNode.Latitude < SplitLatitude && Node.Latitude < SplitLatitude))
				|| (!SplitByLatitude
				&& (PrevNode.Longitude > SplitLongitude && Node.Longitude > SplitLongitude
				|| PrevNode.Longitude < SplitLongitude && Node.Longitude < SplitLongitude)))) {
					// Add node to current polygon
					NewNode = new Object();
					NewNode.Latitude = Node.Latitude;
					NewNode.Longitude = Node.Longitude;
					NewNodes.push(NewNode);
				}

				PrevNode = Node;
			}
		} else
			// This polygon does not require splitting
			Polygons.push(Region[PolygonNum]);
	}

	// Now we have an array of all polygons of source regions, split as necessary
	// Eexplode them into an array of non-intersecting regions
	FirstRegionNum = Regions.length;
	ExplodeRegion(Polygons, Regions);
	
	// Merge inner polygons with outer ones if they have the same border at splitting line
	for (RegionNum = FirstRegionNum; RegionNum < Regions.length; RegionNum++) {
		CurRegion = Regions[RegionNum];
		ParentPolygon = CurRegion[0];
		// If current region was split
		if ((SplitByLatitude && (ParentPolygon.MinLatitude == SplitLatitude || ParentPolygon.MaxLatitude == SplitLatitude))
		|| (!SplitByLatitude && (ParentPolygon.MinLongitude == SplitLongitude || ParentPolygon.MaxLongitude == SplitLongitude))) {
		
			// Collect all children that should be merged
			ChildrenToMerge = new Array();
			for (ChildPolygonNum = 1; ChildPolygonNum < CurRegion.length; ChildPolygonNum++) {
				ChildPolygon = CurRegion[ChildPolygonNum];
				ChildFirstNode = ChildPolygon.Nodes[0];
				ChildLastNode = ChildPolygon.Nodes[ChildPolygon.Nodes.length - 1];
				if ((SplitByLatitude
				&& ChildFirstNode.Latitude == SplitLatitude && ChildLastNode.Latitude == SplitLatitude && ChildFirstNode.Longitude != ChildLastNode.Longitude)
				|| (!SplitByLatitude
				&& ChildFirstNode.Longitude == SplitLongitude && ChildLastNode.Longitude == SplitLongitude && ChildFirstNode.Latitude != ChildLastNode.Latitude)) {
					ChildrenToMerge.push(ChildPolygon);
					CurRegion.splice(ChildPolygonNum, 1);
					ChildPolygonNum--;
				}
			}
			
			if (ChildrenToMerge.length > 0) {
				// Sort children that should be merged
				ChildrenToMerge = ChildrenToMerge.sort(CmpSplitChildren);
				
				// Add children to parent polygon
				while (ChildrenToMerge.length > 0) {
					// Get next child to be added, depending on parent's side and side relatively to split line
					if ((ParentPolygon.Direction == 1
					&& ((SplitByLatitude && ParentPolygon.MaxLatitude == SplitLatitude)
					|| (!SplitByLatitude && ParentPolygon.MinLongitude == SplitLongitude)))
					|| (ParentPolygon.Direction == -1
					&& ((SplitByLatitude && ParentPolygon.MinLatitude == SplitLatitude)
					|| (!SplitByLatitude && ParentPolygon.MaxLongitude == SplitLongitude))))
						ChildPolygon = ChildrenToMerge.pop();
					else
						ChildPolygon = ChildrenToMerge.shift();
						
					// Change child's direction, if necessary
					if (ChildPolygon.Direction == ParentPolygon.Direction)
						ChildPolygon.Nodes.reverse();
						
					// Add child polygon to parent one
					ParentPolygon.Nodes = ParentPolygon.Nodes.concat(ChildPolygon.Nodes);
					ParentPolygon.Area -= ChildPolygon.Area;
				}
				ValidatePolygon(ParentPolygon);
				// Check whether parent polygon became fake
				if (ParentPolygon.Nodes.length < 3) {
					Regions.splice(RegionNum, 1);
					RegionNum--;
				}
			}
		}
	}
}
//-------------------------------------------
function LinkGoesInside(Polygon, StartNodeNum, EndNode) {
	var PrevNode, StartNode, NextNode, PolygonAngleDirection, PrevTriangleDirection, NextTriangleDirection;

	PrevNode = Polygon.Nodes[StartNodeNum > 0 ? StartNodeNum - 1 : Polygon.Nodes.length - 1];
	StartNode = Polygon.Nodes[StartNodeNum];
	NextNode = Polygon.Nodes[StartNodeNum < Polygon.Nodes.length - 1 ? StartNodeNum + 1 : 0];

	PolygonAngleDirection = (StartNode.Latitude - PrevNode.Latitude) * (StartNode.Longitude + PrevNode.Longitude)
	+ (NextNode.Latitude - StartNode.Latitude) * (NextNode.Longitude + StartNode.Longitude)
	+ (PrevNode.Latitude - NextNode.Latitude) * (PrevNode.Longitude + NextNode.Longitude);
	PolygonAngleDirection = PolygonAngleDirection > 0 ? 1 : PolygonAngleDirection < 0 ? -1 :0;

	PrevTriangleDirection = (StartNode.Latitude - PrevNode.Latitude) * (StartNode.Longitude + PrevNode.Longitude)
	+ (EndNode.Latitude - StartNode.Latitude) * (EndNode.Longitude + StartNode.Longitude)
	+ (PrevNode.Latitude - EndNode.Latitude) * (PrevNode.Longitude + EndNode.Longitude);
	PrevTriangleDirection = PrevTriangleDirection > 0 ? 1 : PrevTriangleDirection < 0 ? -1 :0;

	NextTriangleDirection = (StartNode.Latitude - EndNode.Latitude) * (StartNode.Longitude + EndNode.Longitude)
	+ (NextNode.Latitude - StartNode.Latitude) * (NextNode.Longitude + StartNode.Longitude)
	+ (EndNode.Latitude - NextNode.Latitude) * (EndNode.Longitude + NextNode.Longitude);
	NextTriangleDirection = NextTriangleDirection > 0 ? 1 : NextTriangleDirection < 0 ? -1 :0;

	if (PolygonAngleDirection == Polygon.Direction)
		// Polygon's angle is convex. At least one of two triangles must have direction opposite to polygon
		return PrevTriangleDirection == -Polygon.Direction || NextTriangleDirection == -Polygon.Direction;
	else
		// Polygon's angle is concave. Both triangles must have direction opposite to polygon
		return PrevTriangleDirection == -Polygon.Direction && NextTriangleDirection == -Polygon.Direction;
}
//-------------------------------------------
function LinksAreCrossing(Node1, Node2, Node3, Node4) {
	// Checks two links for crossing
	var Numerator1, Numerator2, Denominator, Fraction1, Fraction2;
	
	Numerator1 = (Node4.Latitude - Node3.Latitude) * (Node1.Longitude - Node3.Longitude)
	- (Node4.Longitude - Node3.Longitude) * (Node1.Latitude - Node3.Latitude);
	Numerator2 = (Node2.Latitude - Node1.Latitude) * (Node1.Longitude - Node3.Longitude)
	- (Node2.Longitude - Node1.Longitude) * (Node1.Latitude - Node3.Latitude);
	Denominator = (Node4.Longitude - Node3.Longitude) * (Node2.Latitude - Node1.Latitude)
	- (Node4.Latitude - Node3.Latitude) * (Node2.Longitude - Node1.Longitude);
	if (Denominator != 0) {
		Fraction1 = Numerator1 / Denominator;
		Fraction2 = Numerator2 / Denominator;
		if (Fraction1 >= 0 && Fraction1 <=1 && Fraction2 >= 0 && Fraction2 <=1)
			return true;
		else
			return false;
	} else
		if (Numerator1 == 0)
			if( (Node3.Latitude >= Node1.Latitude && Node3.Latitude <= Node2.Latitude || Node3.Latitude <= Node1.Latitude && Node3.Latitude >= Node2.Latitude)
			&& (Node3.Longitude >= Node1.Longitude && Node3.Longitude <= Node2.Longitude || Node3.Longitude <= Node1.Longitude && Node3.Longitude >= Node2.Longitude))
				return true;
			else
				return false;
		else
			return false;
}
//-------------------------------------------
function LinksAscending(Link1, Link2) {
	return Link1.Length - Link2.Length;
}
//-------------------------------------------
function RegionToPolygon(Region, SourceObjectLineNum, LevelID, RegionsCnt, RegionNum) {
	// Converts region to polygon by merging all holes with border.
	// Assumed that border polygon is the first one and the only one
	var ParentPolygon, ChildPolygonNum, ChildPolygon;
	var ChildNodeNum, ChildNode, ParentNodeNum, ParentNode, LinkLength;
	var LinkIsLegal, PrevNode, NodeNum, Node, PolygonNum, CurrentPolygon;
	var ShortestLinkLength, ClosestChildPolygonNum, MergingPolygon, NewNodes;
	
	if (Region.length <= 1)
		return Region[0];

	var MinLinksToTest = 16384;

	//-------------------------------------------
	// Merge children polygons with parent starting from the one with shortest lisk length
	for (ChildPolygonNum = 1; ChildPolygonNum < Region.length; ChildPolygonNum++) {
		ChildPolygon = Region[ChildPolygonNum];
		ChildPolygon.ShortestLinkLength = Infinity;
		ChildPolygon.ShortestPossibleLinksLengths = new Array();
		for (ChildNodeNum = 0; ChildNodeNum < ChildPolygon.Nodes.length; ChildNodeNum++)
			ChildPolygon.ShortestPossibleLinksLengths[ChildNodeNum] = Infinity;
	}
		
	ParentPolygon = Region[0];
	FirstNewNodeNum = 0;
	LastNewNodeNum = ParentPolygon.Nodes.length - 1;

	PolygonsCnt = Region.length;
	while (Region.length > 1) {
		StatusMessage("Line " + SourceObjectLineNum + ", region at level " + LevelID + ": Splitting " + RegionNum + "/" + RegionsCnt + ", merging " + (PolygonsCnt - Region.length + 1) + "/" + (PolygonsCnt - 1) + "...");

		ClosestChildPolygonNum = undefined;
		ClosestChildNodeNum = undefined;
		ClosestParentNodeNum = undefined;
		ShortestLinkLength = Infinity;
		for (ChildPolygonNum = 1; ChildPolygonNum < Region.length; ChildPolygonNum++)
			ShortestLinkLength = Math.min(ShortestLinkLength, Region[ChildPolygonNum].ShortestLinkLength);
		
		// Find shortest link among children
		for (ChildPolygonNum = 1; ChildPolygonNum < Region.length; ChildPolygonNum++) {
			StatusMessage("Line " + SourceObjectLineNum + ", region at level " + LevelID + ": Splitting " + RegionNum + "/" + RegionsCnt + ", merging " + (PolygonsCnt - Region.length + 1) + "/" + (PolygonsCnt - 1) + " (" + Math.round(ChildPolygonNum * 100 / (Region.length - 1)) + "%)...");
			ChildPolygon = Region[ChildPolygonNum];
			ShortestPossibleLinksLengths = ChildPolygon.ShortestPossibleLinksLengths;
			Links = new Array(); // Here we will store all links for future comparison

			// Find a node in child polygon that could be merged with some node of parent polygon by closest link

			ChildNodesNums = new Array();
			for (ChildNodeNum = 0; ChildNodeNum < ChildPolygon.Nodes.length; ChildNodeNum++)
				ChildNodesNums.push(ChildNodeNum);

			// Collect numbers of other children polygons to an array
			// Later we would pop the number of the most interfering child polygons to the top of array to make checking faster
			OtherChildrenPolygonsNums = new Array();
			for (PolygonNum = 1; PolygonNum < Region.length; PolygonNum++)
				if (PolygonNum != ChildPolygonNum)
					OtherChildrenPolygonsNums.push(PolygonNum);
			
			ChildNodeNum = 0;
			do {
				ChildNodeNum = ChildNodesNums.splice((ChildNodeNum * 106 + 1283) % ChildNodesNums.length, 1)[0];
				ChildNode = ChildPolygon.Nodes[ChildNodeNum];
				//StatusMessage("Line " + SourceObjectLineNum + ", region at level " + LevelID + ": Splitting " + RegionNum + "/" + RegionsCnt + ", merging " + (PolygonsCnt - Region.length + 1) + "/" + (PolygonsCnt - 1) + ", " + ChildPolygonNum + "/" + (Region.length - 1) + ", node " + (ChildPolygon.Nodes.length - ChildNodesNums.length) + "/" + ChildPolygon.Nodes.length + "...");
				// Find a node in parent polygon that could be linked with current child node by shortest link
				if (ShortestPossibleLinksLengths[ChildNodeNum] > ShortestLinkLength) {
					FirstParentNodeNum = FirstNewNodeNum;
					LastParentNodeNum = LastNewNodeNum;
				} else {
					FirstParentNodeNum = 0;
					LastParentNodeNum = ParentPolygon.Nodes.length - 1;
				}
				for (ParentNodeNum = FirstParentNodeNum; ParentNodeNum <= LastParentNodeNum; ParentNodeNum++) {
					ParentNode = ParentPolygon.Nodes[ParentNodeNum];
					// Check current link for being shorter than shortest found
					LinkLength = (ParentNode.Latitude - ChildNode.Latitude) * (ParentNode.Latitude - ChildNode.Latitude)
					+ (ParentNode.Longitude - ChildNode.Longitude) * (ParentNode.Longitude - ChildNode.Longitude); // This is actually a square of link length, but it's enough for comaring
					ShortestPossibleLinksLengths[ChildNodeNum] = Math.min(ShortestPossibleLinksLengths[ChildNodeNum], LinkLength);
					if (LinkLength > ShortestLinkLength || LinkLength == ShortestLinkLength && ClosestChildPolygonNum != undefined)
						continue;
					// Check parent's node for being equal to child's node
					if (ParentNode.Latitude != ChildNode.Latitude
					|| ParentNode.Longitude != ChildNode.Longitude) {
						// Check current link for going in right direction relatively to child polygon
						if (!LinkGoesInside(ChildPolygon, ChildNodeNum, ParentNode))
							continue;
						// Check current link for going in right direction relatively to parent polygon
						if (LinkGoesInside(ParentPolygon, ParentNodeNum, ChildNode))
							continue;
					} else {
						// Check child's links for going inside of parent polygon
						if (LinkGoesInside(ParentPolygon, ParentNodeNum, ChildPolygon.Nodes[ChildNodeNum > 0 ? ChildNodeNum - 1 : ChildPolygon.Nodes.length - 1])
						|| LinkGoesInside(ParentPolygon, ParentNodeNum, ChildPolygon.Nodes[ChildNodeNum < ChildPolygon.Nodes.length - 1 ? ChildNodeNum + 1 : 0]))
							continue;
					}
						
					Link = new Object();
					Link.ChildNodeNum = ChildNodeNum;
					Link.ParentNodeNum = ParentNodeNum;
					Link.Length = LinkLength;
					Links.push(Link);
				}
				// Find shortest legal link
				if (ShortestLinkLength > 0 && (Links.length >= MinLinksToTest || ChildNodesNums.length == 0)) {
					//StatusMessage("Line " + SourceObjectLineNum + ", region at level " + LevelID + ": Splitting " + RegionNum + "/" + RegionsCnt + ", merging " + (PolygonsCnt - Region.length + 1) + "/" + (PolygonsCnt - 1) + ", " + ChildPolygonNum + "/" + (Region.length - 1) + ", node " + (ChildPolygon.Nodes.length - ChildNodesNums.length) + "/" + ChildPolygon.Nodes.length + ", sorting...");
					Links.sort(LinksAscending);
					
					// Place numbers of tests (crossing self, crossing parent, crossing other children) to array
					// Later we would pop the number of the last failed test to the top of array to make checking faster
					TestsNumbers = [0, 2, 1];
					
					for (LinkNum = 0; LinkNum < Links.length; LinkNum++) {
						//StatusMessage("Line " + SourceObjectLineNum + ", region at level " + LevelID + ": Splitting " + RegionNum + "/" + RegionsCnt + ", merging " + (PolygonsCnt - Region.length + 1) + "/" + (PolygonsCnt - 1) + ", " + ChildPolygonNum + "/" + (Region.length - 1) + ", node " + (ChildPolygon.Nodes.length - ChildNodesNums.length) + "/" + ChildPolygon.Nodes.length + ", link " + (LinkNum + 1) + "/" + Links.length + "...");
						Link = Links[LinkNum];
						ChildNode = ChildPolygon.Nodes[Link.ChildNodeNum];
						ParentNode = ParentPolygon.Nodes[Link.ParentNodeNum];

						LinkIsLegal = true;
						if (Link.Length == 0)
							break;

						if (Link.Length > ShortestLinkLength || LinkLength == ShortestLinkLength && ClosestChildPolygonNum != undefined) {
							LinkIsLegal = false;
							break;
						}
							
						for (TestNumber = 0; TestNumber < TestsNumbers.length; TestNumber++) {
							if (TestsNumbers[TestNumber] == 0) {
								// Check link for not crossing any of the child polygon's links except of its neighbouring links
								LinkIsLegal = true;
								PrevNode = ChildPolygon.Nodes[ChildPolygon.Nodes.length - 1];
								for (NodeNum = 0; NodeNum < ChildPolygon.Nodes.length; NodeNum++, PrevNode = Node) {
									Node = ChildPolygon.Nodes[NodeNum];
									if (NodeNum != Link.ChildNodeNum && NodeNum != (Link.ChildNodeNum < ChildPolygon.Nodes.length - 1 ? Link.ChildNodeNum + 1 : 0))
										if (LinksAreCrossing(ChildNode, ParentNode, PrevNode, Node)) {
											LinkIsLegal = false;
											break;
										}
								}
							} else if (TestsNumbers[TestNumber] == 1) {
								// Check link for not crossing any of the parent polygon links except of its neighbouring links
								PrevNode = ParentPolygon.Nodes[ParentPolygon.Nodes.length - 1];
								for (NodeNum = 0; NodeNum < ParentPolygon.Nodes.length; NodeNum++, PrevNode = Node) {
									Node = ParentPolygon.Nodes[NodeNum];
									if (NodeNum != Link.ParentNodeNum && NodeNum != (Link.ParentNodeNum < ParentPolygon.Nodes.length - 1 ? Link.ParentNodeNum + 1 : 0))
										if (LinksAreCrossing(ChildNode, ParentNode, PrevNode, Node)) {
											LinkIsLegal = false;
											break;
										}
								}
							} else if (TestsNumbers[TestNumber] == 2) {
								// Check link for not crossing any of the other child polygons' links
								for (PolygonNum = 0; PolygonNum < OtherChildrenPolygonsNums.length; PolygonNum++) {
									CurrentPolygon = Region[OtherChildrenPolygonsNums[PolygonNum]];
									if (!(Math.min(ChildNode.Latitude, ParentNode.Latitude) > CurrentPolygon.MaxLatitude
									|| Math.max(ChildNode.Latitude, ParentNode.Latitude) < CurrentPolygon.MinLatitude
									|| Math.min(ChildNode.Longitude, ParentNode.Longitude) > CurrentPolygon.MaxLongitude
									|| Math.max(ChildNode.Longitude, ParentNode.Longitude) < CurrentPolygon.MinLongitude)) {
										PrevNode = CurrentPolygon.Nodes[CurrentPolygon.Nodes.length - 1];
										for (NodeNum = 0; NodeNum < CurrentPolygon.Nodes.length; NodeNum++, PrevNode = Node) {
											Node = CurrentPolygon.Nodes[NodeNum];
											if (LinksAreCrossing(ChildNode, ParentNode, PrevNode, Node)) {
												LinkIsLegal = false;
												// Pop this interfering child's number to the top of list
												OtherChildrenPolygonsNums.splice(0, 0, OtherChildrenPolygonsNums.splice(PolygonNum, 1)[0]);
												break;
											}
										}
										if (!LinkIsLegal)
											break;
									}
								}
							}
							if (!LinkIsLegal) {
								TestsNumbers.splice(0, 0, TestsNumbers.splice(TestNumber, 1)[0]);
								break;
							}
						}
						if (!LinkIsLegal)
							continue;
						// Set current link to be shortest
						ClosestChildPolygonNum = ChildPolygonNum;
						ClosestChildNodeNum = Link.ChildNodeNum;
						ClosestParentNodeNum = Link.ParentNodeNum;
						ShortestLinkLength = Link.Length;
						break;
					}
					Links = new Array();
				}
			} while (ChildNodesNums.length > 0 && ShortestLinkLength > 0);
			// If closer link was found in current child, update it
			if (ClosestChildPolygonNum == ChildPolygonNum) {
				ChildPolygon.ShortestLinkLength = ShortestLinkLength;
				ChildPolygon.ClosestNodeNum = ClosestChildNodeNum;
			}
		}
		//-------------------------------------------
		// Merge closest child polygon with parent one
		StatusMessage("Line " + SourceObjectLineNum + ", region at level " + LevelID + ": Splitting " + RegionNum + "/" + RegionsCnt + ", merging " + (PolygonsCnt - Region.length + 1) + "/" + (PolygonsCnt - 1) + "...");
		if (ClosestChildPolygonNum != undefined) {
			MergingPolygon = Region[ClosestChildPolygonNum];
			NewNodes = MergingPolygon.Nodes.slice(ClosestChildNodeNum, MergingPolygon.Nodes.length)
			.concat(MergingPolygon.Nodes.slice(0, ClosestChildNodeNum + 1));
			if (ShortestLinkLength == 0) {
				// Duplicated nodes should be removed, because otherwise
				// we would be unable to check direction of link starting at one of these nodes
				// Although, we would not remove redundand nodes at this point, we would do that when all children are merged
				NewNodes.splice(0, 1);
				NewNodes.pop();
			}
			if (MergingPolygon.Direction == ParentPolygon.Direction)
				NewNodes.reverse();
			ParentPolygon.Nodes = ParentPolygon.Nodes.slice(0, ClosestParentNodeNum + 1)
			.concat(NewNodes, ParentPolygon.Nodes.slice(ClosestParentNodeNum, ParentPolygon.Nodes.length));

			FirstNewNodeNum = ClosestParentNodeNum;
			LastNewNodeNum = ClosestParentNodeNum + NewNodes.length + 1;
			// Add child's source polygons' numbers to parent polygon
			for (PolygonNum = 0; PolygonNum < MergingPolygon.SourcePolygons.length; PolygonNum++)
				if (ArrayIndexOf(ParentPolygon.SourcePolygons, MergingPolygon.SourcePolygons[PolygonNum]) == -1)
					ParentPolygon.SourcePolygons.push(MergingPolygon.SourcePolygons[PolygonNum]);
			// Remove merged child polygon from array
			Region.splice(ClosestChildPolygonNum, 1);
		} else {
			// We were unable to find closest child among existing children
			// Let's drop one of them, maybe, it will be possible with less children
			ErrorMessage("Error: Line " + SourceObjectLineNum + ", region at level " + LevelID
			+ ": Cannot merge polygon made of polygon" + (Region[1].SourcePolygons.length == 1 ? " " : "s ") + Region[1].SourcePolygons.join(", ")
			+ " into polygon made of polygon" + (Region[0].SourcePolygons.length == 1 ? " " : "s ") + Region[0].SourcePolygons.join(", ") + "! Polygon dropped.");
			Region.splice(1, 1);
		}
	}
	//-------------------------------------------
	StatusMessage("Line " + SourceObjectLineNum + ", region at level " + LevelID + ": Splitting " + RegionNum + "/" + RegionsCnt + ", validating...");
	ValidatePolygon(ParentPolygon);
	return ParentPolygon;
}
//-------------------------------------------
function ValidateSection(Section) {
	var Nodes, NodeNum;

	Nodes = Section.Nodes;
	// Delete fake nodes between first and last nodes
	for (NodeNum = 1; NodeNum < Nodes.length; NodeNum++) {
		if (Nodes[NodeNum - 1].Latitude == Nodes[NodeNum].Latitude
		&& Nodes[NodeNum - 1].Longitude == Nodes[NodeNum].Longitude) {
			Nodes.splice(NodeNum, 1);
			NodeNum--;
		} else if(NodeNum > 1
		&& PointWithinLink(Nodes[NodeNum - 1], Nodes[NodeNum - 2], Nodes[NodeNum])) {
			Nodes.splice(NodeNum - 1, 1);
			NodeNum -= 2;
		}
	}
	SetSectionProperties(Section);
}
//-------------------------------------------
function ValidateRegion(Region, SourceObjectLineNum, LevelID) {
	// Validates region
	// Currently only deletes duplicated polygons
	var ThereWereFakeNodes, PolygonNum, Polygon, Nodes, NodeNum, PrevNode, Node, NextNode;
	var PolygonsToDelete, OriginalPolygonNum, Duplicates, DuplicatePolygonNum, PolygonNumID;
	var Message, Padding, CharNum;
	
	// Validate polygons (delete fake nodes, delete fake polygons)
	for (PolygonNum = 0; PolygonNum < Region.length; PolygonNum++) {
		StatusMessage("Line " + SourceObjectLineNum + ", region at level " + LevelID + ": Validating polygons " + (PolygonNum + 1 ) + "/" + Region.length + "...");
		Polygon = Region[PolygonNum];
		ValidatePolygon(Polygon);
		if (Polygon.Nodes.length < 3) {
			if (CommandLineMode()) {
				WarningMessage("Warning: Line " + SourceObjectLineNum + ", region at level " + LevelID
				+ ": Polygon " + Polygon.SourcePolygons[0] + " is fake");
			}
			Region.splice(PolygonNum, 1);
			PolygonNum--;
		}
	}
	
	// Delete duplicated polygons
	PolygonsToDelete = new Array();
	for (OriginalPolygonNum = 0; OriginalPolygonNum < Region.length - 1; OriginalPolygonNum++) {
		StatusMessage("Line " + SourceObjectLineNum + ", region at level " + LevelID + ": Checking for duplicated polygons " + (OriginalPolygonNum + 1 ) + "/" + Region.length + "...");
		if (ArrayIndexOf(PolygonsToDelete, OriginalPolygonNum) != -1)
			continue; // Polygon is already a duplicate of some earlier polygon
		Duplicates = new Array();
		for (DuplicatePolygonNum = OriginalPolygonNum + 1; DuplicatePolygonNum < Region.length; DuplicatePolygonNum++)
			if (Region[DuplicatePolygonNum].MinLatitude == Region[OriginalPolygonNum].MinLatitude
			&& Region[DuplicatePolygonNum].MinLongitude == Region[OriginalPolygonNum].MinLongitude
			&& Region[DuplicatePolygonNum].MaxLatitude == Region[OriginalPolygonNum].MaxLatitude
			&& Region[DuplicatePolygonNum].MaxLongitude == Region[OriginalPolygonNum].MaxLongitude
			&& PolygonWithinPolygon(Region[DuplicatePolygonNum], Region[OriginalPolygonNum]) == 0)
				Duplicates.push(DuplicatePolygonNum);
		if (Duplicates.length > 0) {
			if (CommandLineMode()) {
				Message = "Warning: Line " + SourceObjectLineNum + ", region at level " + LevelID
				+ ": Polygon " + Region[OriginalPolygonNum].SourcePolygons[0] + " is duplicated by polygon"
				+ (Duplicates.length == 1 ? "" : "s ");
				for (DuplicatePolygonNum = 0; DuplicatePolygonNum < Duplicates.length; DuplicatePolygonNum++)
					Message += (DuplicatePolygonNum == 0 ? "" : ", ") + Region[Duplicates[DuplicatePolygonNum]].SourcePolygons[0];
				Message += ".";
				WarningMessage(Message);
			}
			if (Duplicates.length % 2 == 1)
				PolygonsToDelete.push(OriginalPolygonNum);
			for (PolygonNumID in Duplicates)
				PolygonsToDelete.push(Duplicates[PolygonNumID]);
		}
	}
	for (PolygonNumID = PolygonsToDelete.length - 1; PolygonNumID >= 0; PolygonNumID--)
		Region.splice(PolygonsToDelete[PolygonNumID], 1);
}
//-------------------------------------------
function ValidatePolyline(Polyline, SourceObjectLineNum, LevelID) {
	var LineNum, Line, Message, Padding;

	// Validate sections (delete fake nodes, delete fake sections)
	for (LineNum = 0; LineNum < Polyline.length; LineNum++) {
		StatusMessage("Line " + SourceObjectLineNum + ", polyline at level " + LevelID + ": Validating lines " + (LineNum + 1 ) + "/" + Polyline.length + "...");
		Line = Polyline[LineNum];
		ValidateSection(Line);
		if (Line.Nodes.length < 2) {
			if (CommandLineMode()) {
				WarningMessage("Warning: Line " + SourceObjectLineNum + ", polyline at level " + LevelID
				+ ": Line " + Line.SourceSections[0] + " is fake");
			}
			Polyline.splice(LineNum, 1);
			LineNum--;
		}
	}
}
//-------------------------------------------
function ExportNode(Node) {
	return "(" + Node.Latitude.toFixed(CoordinatesPrecision) + "," + Node.Longitude.toFixed(CoordinatesPrecision) + ")";
}
//-------------------------------------------
function ExportPolygon(Polygon) {
	var PolygonString, NodeNum;

	PolygonString = "";
	for (NodeNum = 0; NodeNum < Polygon.Nodes.length; NodeNum++)
		PolygonString += ExportNode(Polygon.Nodes[NodeNum]) + ",";
		
	if (PolygonString.length > 0)
		PolygonString = PolygonString.substr(0, PolygonString.length - 1);
		
	return PolygonString;
}
//-------------------------------------------
function ExportSection(Section) {
	var SectionString, NodeNum;

	SectionString = "";
	for (NodeNum = 0; NodeNum < Section.Nodes.length; NodeNum++)
		SectionString += ExportNode(Section.Nodes[NodeNum]) + ",";
		
	if (SectionString.length > 0)
		SectionString = SectionString.substr(0, SectionString.length - 1);
		
	return SectionString;
}
//-------------------------------------------
function FormatBlockID(BlockID) {
	if (BlockIdFromatIsHex)
		return "I" + FillLeft(BlockID.toString(16).toUpperCase(), 7, "0");
	else
		return FillLeft(BlockID.toString(), 8, "0");
}
//-------------------------------------------
function BlockNameByID(BlockID) {
	var ID, Lat, Lon, BlockName, Hemisphere;

	if (BlockID < 2593) {
		ID = BlockID - 1;
		Lon = ID % 180 * 2 - 180;
		Lat = (ID - ID % 180) / 180 * 2;
		Lat = (Lat < 90 ? Lat : 88 - Lat);
		BlockName = (Lat >= 0 ? "N" + Lat + "-" + (Lat + 2) : "S" + (- Lat - 2) + "-" + (- Lat))
		+ ", " + (Lon >= 0 ? "E" + Lon + "-" + (Lon + 2) : "W" + (- Lon - 2) + "-" + (- Lon));
	} else {
		ID = BlockID - 2593;
		if (ID < 198420)
			Hemisphere = 1;
		else {
			Hemisphere = -1;
			ID -= 198420;
		}
		SubsheetID = ID % 185;
		SheetID = (ID - SubsheetID) / 185;
		if (SheetID < 900) { // Latitude <= 60
			Lon = SheetID % 60;
			Lat = (SheetID - Lon) / 60;
		} else if (SheetID < 1020) { // Latitude <= 76
			Lon = (SheetID - 900) % 30;
			Lat = 15 + (SheetID - 900 - Lon) / 30;
		}
		//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
	}
	return BlockName;
}
//-------------------------------------------
function BlockIDByName(BlockName) {
	var match, ID;
	
	match = BlockName.toUpperCase().match(/^(N|S)(\d+)-(\d+), (E|W)(\d+)-(\d+)$/);
	if (match != null) {
		// Split map: 2 by longitude, 2 by latitude (like WorldMap)
		Lat = (match[1] == "S" ? 90 : 0) + match[2];
		Lon = (match[4] = "W" ? 180 - match[5] : 180 + match[5]);
		ID = Lat * 180 + Lon;
	}
	//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
}
//-------------------------------------------
function AddBlockID(Blocks, Latitude, Longitude, Options) {
	var BlockID, BlockName;

	if (Options.SplitMode == 1) {
		BlockID = 1 + (Latitude >= 0 ? Latitude : 85 - Latitude) * 180 / 2
		+ (Longitude + 180) / 2;
		BlockName = (Latitude >= 0 ? "N" + Latitude + "-" + (Latitude + 2) : "S" + (- Latitude - 2) + "-" + (- Latitude))
		+ ", " + (Longitude >= 0 ? "E" + Longitude + "-" + (Longitude + 2) : "W" + (- Longitude - 2) + "-" + (- Longitude));
	} else if (Options.SplitMode > 1) {
	
		// Calculate block name
		BlockName = Latitude >= 0 ? String.fromCharCode("A".charCodeAt(0) + Math.floor(Latitude / 4 + o))
		: "S-" + String.fromCharCode("A".charCodeAt(0) - 1 - Math.floor(Latitude / 4 + o));
		if (Options.SplitMode == 2) {
			if (Latitude >= -60 && Latitude < 60)
				BlockName += "-" + ((Longitude + 180) / 6 + 1);
			else if (Latitude >= -76 && Latitude < 76)
				BlockName += "-" + ((Longitude + 180) / 6 + 1)
				+ "," + ((Longitude + 180) / 6 + 2);
			else
				BlockName += "-" + ((Longitude + 180) / 6 + 1)
				+ "," + ((Longitude + 180) / 6 + 2)
				+ "," + ((Longitude + 180) / 6 + 3)
				+ "," + ((Longitude + 180) / 6 + 4);
		} else if (Options.SplitMode == 3) {
			if (Latitude >= -60 && Latitude < 60)
				BlockName += "-" + (Math.floor((Longitude + 180) / 6 + o) + 1)
				+ "-" + String.fromCharCode("A".charCodeAt(0) + Math.floor((Longitude + 180) % 6 / 3 + o) + (1 - Math.floor((Latitude + 92) % 4 / 2 + o)) * 2);
			else if (Latitude >= -76 && Latitude < 76)
				BlockName += "-" + (Math.floor((Longitude + 180) / 6 + o) + 1)
				+ "-" + String.fromCharCode("A".charCodeAt(0) + Math.floor((Longitude + 180) % 6 / 3 + o) + 0 + (1 - Math.floor((Latitude + 92) % 4 / 2 + o)) * 2)
				+ "," + String.fromCharCode("A".charCodeAt(0) + Math.floor((Longitude + 180) % 6 / 3 + o) + 1 + (1 - Math.floor((Latitude + 92) % 4 / 2 + o)) * 2);
			else
				BlockName += "-" + (Math.floor((Longitude + 180) / 6 + o) + 1)
				+ "," + (Math.floor((Longitude + 186) / 6 + o) + 1)
				+ "-" + String.fromCharCode("A".charCodeAt(0) + Math.floor((Longitude + 180) % 6 / 3 + o) + 0 + (1 - Math.floor((Latitude + 92) % 4 / 2 + o)) * 2)
				+ "," + String.fromCharCode("A".charCodeAt(0) + Math.floor((Longitude + 180) % 6 / 3 + o) + 1 + (1 - Math.floor((Latitude + 92) % 4 / 2 + o)) * 2);
		} else if (Options.SplitMode == 4) {
			BlockName += "-" + (Math.floor((Longitude + 180) / 6 + o) + 1);
			if (Latitude >= -60 && Latitude < 60)
				BlockName += "-" + Roman(Math.floor((Longitude + 180) % 6 + o) + 1 + (5 - Math.floor((Latitude + 92) % 4 * 3 / 2 + o)) * 6);
			else if (Latitude >= -76 && Latitude < 76)
				BlockName += "-" + Roman(Math.floor((Longitude + 180) % 6 + o) + 1 + (5 - Math.floor((Latitude + 92) % 4 * 3 / 2 + o)) * 6)
				+ "," + Roman(Math.floor((Longitude + 180) % 6 + o) + 2 + (5 - Math.floor((Latitude + 92) % 4 * 3 / 2 + o)) * 6);
			else
				BlockName += "-" + Roman(Math.floor((Longitude + 180) % 6 + o) + 1 + (5 - Math.floor((Latitude + 92) % 4 * 3 / 2 + o)) * 6)
				+ "," + Roman(Math.floor((Longitude + 180) % 6 + o) + 2 + (5 - Math.floor((Latitude + 92) % 4 * 3 / 2 + o)) * 6)
				+ (Math.floor((Longitude + 180) % 6 + o) != 4
				? "," + Roman(Math.floor((Longitude + 180) % 6 + o) + 3 + (5 - Math.floor((Latitude + 92) % 4 * 3 / 2 + o)) * 6)
				+ "," + Roman(Math.floor((Longitude + 180) % 6 + o) + 4 + (5 - Math.floor((Latitude + 92) % 4 * 3 / 2 + o)) * 6)
				: ";" + (Latitude >= 0 ? String.fromCharCode("A".charCodeAt(0) + Math.floor(Latitude / 4 + o))
				: "S-" + String.fromCharCode("A".charCodeAt(0) - 1 - Math.floor(Latitude / 4 + o)))
				+ "-" + (Math.floor((Longitude + 183) / 6 + o) + 1)
				+ "-" + Roman(Math.floor((Longitude + 180) % 6 + o) - 3 + (5 - Math.floor((Latitude + 92) % 4 * 3 / 2 + o)) * 6)
				+ "," + Roman(Math.floor((Longitude + 180) % 6 + o) - 2 + (5 - Math.floor((Latitude + 92) % 4 * 3 / 2 + o)) * 6));
		} else { // if (Options.SplitMode == 5) {
			BlockName += "-" + (Math.floor((Longitude + 180) / 6 + o) + 1);
			if (Latitude >= -60 && Latitude < 60)
				BlockName += "-" + (Math.ceil((Longitude + 180) % 6 * 2 - o) + 1 + (11 - Math.ceil((Latitude + 92) % 4 * 3 - o)) * 12);
			else if (Latitude >= -76 && Latitude < 76)
				BlockName += "-" + (Math.ceil((Longitude + 180) % 6 * 2 - o) + 1 + (11 - Math.ceil((Latitude + 92) % 4 * 3 - o)) * 12)
				+ "," + (Math.ceil((Longitude + 180) % 6 * 2 - o) + 2 + (11 - Math.ceil((Latitude + 92) % 4 * 3 - o)) * 12);
			else
				BlockName += "-" + (Math.ceil((Longitude + 180) % 6 * 2 - o) + 1 + (11 - Math.ceil((Latitude + 92) % 4 * 3 - o)) * 12)
				+ "," + (Math.ceil((Longitude + 180) % 6 * 2 - o) + 2 + (11 - Math.ceil((Latitude + 92) % 4 * 3 - o)) * 12)
				+ "," + (Math.ceil((Longitude + 180) % 6 * 2 - o) + 3 + (11 - Math.ceil((Latitude + 92) % 4 * 3 - o)) * 12)
				+ "," + (Math.ceil((Longitude + 180) % 6 * 2 - o) + 4 + (11 - Math.ceil((Latitude + 92) % 4 * 3 - o)) * 12);
		}

		// Calculate block ID
		BlockID = 2593;
		if (Latitude >= 0)
			Lat = Math.floor(Latitude / 4 + o);
		else {
			Lat = Math.ceil(-Latitude / 4 - o) - 1;
			BlockID += 198420;
		}
		BlockID += 185 * (Math.min(15, Lat) * 60
		+ Math.min(4, Math.max(Lat - 15, 0)) * 30
		+ Math.min(3, Math.max(Lat - 19, 0)) * 15
		+ (Lat < 15
		? Math.floor((180 + Longitude) / 6 + o)
		: Lat < 19
		? Math.floor((180 + Longitude) / 12 + o)
		: Lat < 22
		? Math.floor((180 + Longitude) / 24 + o) : 0))
		+ 93 * (Lat == 22
		? Math.floor((180 + Longitude) / 24 + o) : 0);

		Lat = Math.round((92 + Latitude) * 3) % 12;
		Lon = Math.round(Latitude < 60 && Latitude >= -60 ? (180 + Longitude) * 2
		: Latitude < 76 && Latitude >= -76 ? 180 + Longitude
		: (180 + Longitude) / 2) % 12;
		
		if (Options.SplitMode >= 2)
			BlockID++;
		if (Options.SplitMode >= 3)
			BlockID += (Math.floor(Lon / 6  + o)
			+ (1 - Math.floor(Lat / 6 + o)) * 2) * 46 + 1;
		if (Options.SplitMode >= 4)
			BlockID += (Math.floor(Lon % 6 / 2  + o)
			+ (2 - Math.floor(Lat % 6 / 2 + o)) * 3) * 5 + 1;
		if (Options.SplitMode == 5)
			BlockID += Lon % 2
			+ (1 - Lat % 2) * 2 + 1;
	}
	
//if (BlockNameByID(BlockID) != BlockName) {
//	ErrorMessage("Block name for block " + BlockID + " should be " + BlockName + ", but it's " + BlockNameByID(BlockID));
//}
	
	// Check whether current block should be skipped (/skipblocks option)
	if (Options.BlocksToSkip[BlockName.toLowerCase()] || Options.BlocksToSkip[String(BlockID ^ BaseBlockID)])
		return; // This block should be skipped
	
	// Check whether current block should be sticked to another block (/stickblocks option)
	ReplacementBlock = Options.BlocksToStick[String(BlockID ^ BaseBlockID)];
	if (ReplacementBlock == undefined)
		ReplacementBlock = Options.BlocksToStick[BlockName.toLowerCase()];
	if (typeof(ReplacementBlock) == "number") {
		BlockID = ReplacementBlock ^ BaseBlockID;
		BlockName = BlockNameByID(BlockID);
	} else if (typeof(ReplacementBlock) == "string") {
		BlockName = ReplacementBlock;
		BlockID = BlockIDByName(BlockID);
	}
	
	Blocks.length++;
	Blocks[BlockName] = BlockID;
}
//-------------------------------------------
function AddBlocksForSection(Blocks, MinLatitude, MinLongitude, MaxLatitude, MaxLongitude, Options) {
	var MinStep, MinBlockLatitude, MaxBlockLatitude, MinBlockLongitude, MaxBlockLongitude, Blocks;

	MinStep = 0.25;
	if (Options.SplitMode == 1) {
		MinBlockLatitude = Math.floor(MinLatitude / 2 + o) * 2;
		MaxBlockLatitude = Math.ceil(MaxLatitude / 2 - o) * 2;
		if (MinBlockLatitude == 90)
			MinBlockLatitude -= 2;
		MinBlockLongitude = Math.floor(MinLongitude / 2 + o) * 2;
		MaxBlockLongitude = Math.ceil(MaxLongitude / 2 - o) * 2;
	} else {
		SplitScale = Options.SplitMode == 2 ? 1 : Options.SplitMode == 3 ? 2 : Options.SplitMode == 4 ? 6 : 12;
		MinBlockLatitude = Math.floor(MinLatitude * SplitScale / 4 + o) * 4 / SplitScale;
		MaxBlockLatitude = Math.ceil(MaxLatitude * SplitScale / 4 - o) * 4 / SplitScale;
		if (MinBlockLatitude == 90)
			MinBlockLatitude -= 4 / SplitScale;
		if (MaxLatitude <= 60 && MinLatitude >= -60) {
			MinBlockLongitude = Math.floor((MinLongitude + 180) * SplitScale / 6 + o) * 6 / SplitScale - 180;
			MaxBlockLongitude = Math.ceil((MaxLongitude + 180) * SplitScale / 6 - o) * 6 / SplitScale - 180;
			if (MinBlockLongitude == 180)
				MinBlockLongitude -= 6 / SplitScale;
		} else if (MaxLatitude <= 76 && MinLatitude >= -76) {
			MinBlockLongitude = Math.floor((MinLongitude + 180) * SplitScale / 12 + o) * 12 / SplitScale - 180;
			MaxBlockLongitude = Math.ceil((MaxLongitude + 180) * SplitScale / 12 - o) * 12 / SplitScale - 180;
			if (MinBlockLongitude == 180)
				MinBlockLongitude -= 12 / SplitScale;
		} else {
			MinBlockLongitude = Math.floor((MinLongitude + 180) * SplitScale / 24 + o) * 24 / SplitScale - 180;
			MaxBlockLongitude = Math.ceil((MaxLongitude + 180) * SplitScale / 24 - o) * 24 / SplitScale - 180;
			if (MinBlockLongitude == 180)
				MinBlockLongitude -= 24 / SplitScale;
		}
	}
	
	AddBlockID(Blocks, MinBlockLatitude, MinBlockLongitude, Options);
	// Add additional blocks if object is on some borders
	// We would not currently do this. Maybe, in future versions.
	//if (MinBlockLatitude == MaxBlockLatitude && MinBlockLatitude > -90) {
	//	if (MinBlockLongitude == MaxBlockLongitude && MinBlockLongitude > -180) {
	//		AddBlocksForSection(Blocks, MinBlockLatitude - MinStep, MinBlockLongitude, MaxBlockLatitude, MaxBlockLatitude, Options);
	//		AddBlocksForSection(Blocks, MinBlockLatitude, MinBlockLongitude - MinStep, MaxBlockLatitude + MinStep, MaxBlockLatitude, Options);
	//	} else
	//		AddBlocksForSection(Blocks, MinBlockLatitude - MinStep, MinBlockLongitude, MaxBlockLatitude, MaxBlockLatitude, Options);
	//} else if (MinBlockLongitude == MaxBlockLongitude && MinBlockLongitude > -180)
	//	AddBlocksForSection(Blocks, MinBlockLatitude, MinBlockLongitude - MinStep, MaxBlockLatitude, MaxBlockLatitude, Options);
	
	return Blocks;
}
//-------------------------------------------
function ProcessDestinationString(String, Options) {

	if (DestinationFile == undefined && MapSection == undefined && ImgSection == undefined)
		FileHeader.push(String);
	else if (Options == undefined || Options.SplitMode == 0)
		DestinationFile.WriteLine(String);
}
//-------------------------------------------
function ProcessDestinationSection(Section, MinLatitude, MinLongitude, MaxLatitude, MaxLongitude, Options) {
	var LineNum, MapFileName, MapFileNameNumber, ProductCode;

	if (Options == undefined || Options.SplitMode == 0) {
		if (DestinationFile == undefined) {
			DestinationFile = fso.OpenTextFile(DestinationFileOrFolderName, ForWriting, true);
			DestinationFile.WriteLine("; Processed with pGPSmap " + sVersion + " (www.olexa.com.ua/gps/pGPSmap)");
			DestinationFile.WriteLine();
			for (LineNum = 0; LineNum < FileHeader.length; LineNum++)
				DestinationFile.WriteLine(FileHeader[LineNum]);
		}
		for (LineNum = 0; LineNum < Section.length; LineNum++)
			if (Trim(Section[LineNum].substr(0, Section[LineNum].indexOf("="))).toLowerCase() == "datum") {
				if (ImgParams["datum"].toUpperCase() == "SK-42")
					ProcessDestinationString(Section[LineNum].substr(0, Section[LineNum].indexOf("=") + 1) + "W84");
				else
					ProcessDestinationString(Section[LineNum]);
			} else
				ProcessDestinationString(Section[LineNum]);
		return;
	}

	if (DestinationFile == undefined) {

		// Determine map file name
		if (MapParams["filename"] != undefined)
			MapFileName = MapParams["filename"];
		else if (ImgParams["name"] != undefined) {
			MapFileName = ImgParams["name"];
			WarningMessage("Warning: FileName value in [Map] section is missing!\n"
			+"Name value from [IMG ID] section is used as map file name.");
		} else {
			MapFileName = fso.GetBaseName(SourceFileName);
			WarningMessage("Warning: FileName value in [Map] section and Name value in [IMG ID] section are missing!\n"
			+"Source file name is used as map file name.");
		}
		MapFileName = MapFileName.replace("\\"," ").replace("/"," ").replace(":"," ").replace("*"," ").replace("?"," ").replace("<"," ").replace(">"," ").replace("|"," ");
		while (MapFileName.indexOf("  ") != -1)
			MapFileName = MapFileName.replace("  ", "");
			
		// Generate number that represents map file name for generating product code and base map ID
		for (MapFileNameNumber = 0, Pos = 0; Pos < MapFileName.length; Pos++)
			MapFileNameNumber += (MapFileName.charCodeAt(Pos) < 0x20 ? 0
			: MapFileName.charCodeAt(Pos) & 0x7f < 0x22 ? MapFileName.charCodeAt(Pos) & 0x7f - 0x20
			: MapFileName.charCodeAt(Pos) & 0x7f < 0x2a ? MapFileName.charCodeAt(Pos) & 0x7f - 0x21
			: MapFileName.charCodeAt(Pos) & 0x7f < 0x2f ? MapFileName.charCodeAt(Pos) & 0x7f - 0x22
			: MapFileName.charCodeAt(Pos) & 0x7f < 0x3a ? MapFileName.charCodeAt(Pos) & 0x7f - 0x23
			: MapFileName.charCodeAt(Pos) & 0x7f < 0x3c ? MapFileName.charCodeAt(Pos) & 0x7f - 0x24
			: MapFileName.charCodeAt(Pos) & 0x7f < 0x3e ? MapFileName.charCodeAt(Pos) & 0x7f - 0x25
			: MapFileName.charCodeAt(Pos) & 0x7f < 0x3f ? MapFileName.charCodeAt(Pos) & 0x7f - 0x26
			: MapFileName.charCodeAt(Pos) & 0x7f < 0x5c ? MapFileName.charCodeAt(Pos) & 0x7f - 0x27
			: MapFileName.charCodeAt(Pos) & 0x7f < 0x7c ? MapFileName.charCodeAt(Pos) & 0x7f - 0x28
			: MapFileName.charCodeAt(Pos) & 0x7f - 0x29) * Math.pow(87, Pos);

		// Generate product code, if necessary
		if (MapParams["productcode"] == undefined) {
			ProductCode = MapFileNameNumber % (255 - 40) + 41;
			WarningMessage("Warning: ProductCode value in [Map] section is missing!\n"
			+"Generated basing on map file name: " + ProductCode);
		}

		// Open map file
		DestinationFile = fso.OpenTextFile(fso.BuildPath(DestinationFileOrFolderName, MapFileName + ".bat"), ForWriting, true);
		DestinationFile.WriteLine("REM batchfile fr cgpsmapper");
	
		
		// Get map block ID format
		BlockIdFromatIsHex = ImgParams["id"] != undefined && ImgParams["id"].substr(0, 1).toUpperCase() == "I";
		
		// Get or generate base map block ID
		if ((BaseBlockID = ImgParams["id"], BaseBlockID == undefined)
		|| (BlockIdFromatIsHex ? BaseBlockID = parseInt(BaseBlockID.substr(1), 16) : BaseBlockID = parseInt(BaseBlockID, 10)
		, isNaN(BaseBlockID) || BaseBlockID == 0 || BaseBlockID == 1 || BaseBlockID == 10000001)) {
			// Generate map block ID from map file name
			BaseBlockID = BlockIdFromatIsHex ? MapFileNameNumber % (0xfffffff - 30000000) + 30000001
			: MapFileNameNumber % (0x5e7ffff - 30000000) + 30000001
			WarningMessage("Warning: ID value in [IMG ID] section is missing or degraded!\n"
			+"Generated basing on map file name: " + FormatBlockID(BaseBlockID));
		}
		
		// Create array for map blocks files
		BlockFiles = new Array();
		PendingSections = new Array();
	}

	// Find blocks for current section
	Blocks = new Array();
	AddBlocksForSection(Blocks, MinLatitude, MinLongitude, MaxLatitude, MaxLongitude, Options);
	
	if (Blocks.length == 0)
		return; // Current section's block should be skipped
	
	// Find block file for current section and write section there
	for (BlockFileNum = 0; BlockFileNum < BlockFiles.length; BlockFileNum++)
		// If current block file's block is within section's blocks
		if (Blocks[BlockFiles[BlockFileNum].Name] != undefined) {
			// Write section to current block file
			BlockFiles[BlockFileNum].File.WriteLine();
			for (LineNum = 0; LineNum < Section.length; LineNum++)
				BlockFiles[BlockFileNum].File.WriteLine(Section[LineNum]);
			return;
		}

	// There's no block file for current section yet
	// If section is covered by one block only, create new block file
	//if (Blocks.length == 1) {
	
	// Create new block file
	BlockFile = new Object();
	for (BlockName in Blocks) {
		BlockFile.ID = Blocks[BlockName] ^ BaseBlockID;
		BlockFile.Name = BlockName;
	}                
	DestinationFile.WriteLine('"C:\\Program Files (x86)\\cGPSmapper\\"cgpsmapper "' + FormatBlockID(BlockFile.ID) + '.mp" -q');
	BlockFile.File = fso.OpenTextFile(fso.BuildPath(DestinationFileOrFolderName, FormatBlockID(BlockFile.ID) + ".mp"), ForWriting, true, TristateFalse);

	BlockFile.File.WriteLine();
	for (LineNum = 0; LineNum < ImgSection.length; LineNum++) {
		if (LineNum == 1 && ImgParams["id"] == undefined)
			BlockFile.File.WriteLine("ID=" + FormatBlockID(BlockFile.ID));
		if (LineNum == 1 && ImgParams["name"] == undefined)
			BlockFile.File.WriteLine("Name=" + BlockFile.Name
			+ (Options.Names[String(BlockFile.ID)] != undefined ? " " + Options.Names[String(BlockFile.ID)]
			: (Options.Names[String(BlockFile.Name)] != undefined ? " " + Options.Names[String(BlockFile.Name)] : "")));
		if (Trim(ImgSection[LineNum].substr(0, ImgSection[LineNum].indexOf("="))).toLowerCase() == "id")
			BlockFile.File.WriteLine(ImgSection[LineNum].substr(0, ImgSection[LineNum].indexOf("=") + 1) + FormatBlockID(BlockFile.ID));
		else if (Trim(ImgSection[LineNum].substr(0, ImgSection[LineNum].indexOf("="))).toLowerCase() == "name")
			BlockFile.File.WriteLine(ImgSection[LineNum].substr(0, ImgSection[LineNum].indexOf("=") + 1) + BlockFile.Name
			+ (Options.Names[String(BlockFile.ID)] != undefined ? " " + Options.Names[String(BlockFile.ID)]
			: (Options.Names[String(BlockFile.Name)] != undefined ? " " + Options.Names[String(BlockFile.Name)] : "")));
		else if (Trim(ImgSection[LineNum].substr(0, ImgSection[LineNum].indexOf("="))).toLowerCase() == "datum") {
			if (ImgParams["datum"].toUpperCase() == "SK-42")
				BlockFile.File.WriteLine(ImgSection[LineNum].substr(0, ImgSection[LineNum].indexOf("=") + 1) + "W84");
			else
				BlockFile.File.WriteLine(ImgSection[LineNum]);
		} else
			BlockFile.File.WriteLine(ImgSection[LineNum]);
	}

	// Write pending sections that are covered with newly created file's block into this file
	for (SectionNum = 0; SectionNum < PendingSections.length; SectionNum++) {
		PendingSection = PendingSections[SectionNum];
		if (PendingSection.Blocks[BlockFile.Name] != undefined) {
			BlockFile.File.WriteLine();
			for (LineNum = 0; LineNum < PendingSection.Source.length; LineNum++)
				BlockFile.File.WriteLine(PendingSection.Source[LineNum]);
			PendingSections.splice(SectionNum, 1);
			SectionNum--;
		}
	}

	// Write current section to newly created file
	BlockFile.File.WriteLine();
	for (LineNum = 0; LineNum < Section.length; LineNum++)
		BlockFile.File.WriteLine(Section[LineNum]);
		
	// Add created block file to block files list
	BlockFiles.push(BlockFile);
	
	//}
	
	// As we currently use only one block for section, we would never get here.
	
}
//-------------------------------------------
function ProcessDestinationPolygon(Polygon, LevelID, CommonElements, Options) {
	var ElementID;
	
	DestinationSection = new Array();
	DestinationSection.push("[RGN80]");
	for (ElementID in CommonElements)
		DestinationSection.push(CommonElements[ElementID]);
	DestinationSection.push("Data" + LevelID + "=" + ExportPolygon(Polygon));
	DestinationSection.push("[END-RGN80]");
	DestinationSection.push("");
	PolygonsCreated++;

	ProcessDestinationSection(DestinationSection, Polygon.MinLatitude, Polygon.MinLongitude, Polygon.MaxLatitude, Polygon.MaxLongitude, Options);
}
//-------------------------------------------
function ProcessDestinationLine(Line, LevelID, CommonElements, Options) {
	var ElementID;
	
	DestinationSection = new Array();
	DestinationSection.push("[RGN40]");
	for (ElementID in CommonElements)
		DestinationSection.push(CommonElements[ElementID]);
	DestinationSection.push("Data" + LevelID + "=" + ExportSection(Line));
	DestinationSection.push("[END-RGN40]");
	DestinationSection.push("");
	LinesCreated++;

	ProcessDestinationSection(DestinationSection, Line.MinLatitude, Line.MinLongitude, Line.MaxLatitude, Line.MaxLongitude, Options);
}
//-------------------------------------------
function CutPolygonByPolygon(Polygon, CuttingPolygon, DestRegion, GetInnerPart) {
	//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
}
//-------------------------------------------
function CutRegionByRegion(Region, CuttingRegion) {
	var CutRegion, CuttingRegionNum, CuttingRegion, PolygonNum;
	var FirstPolygonNum, LastPolygonNum, CutPolygonNum;
	
	//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
	CutRegion = Region;
	return CutRegion;
	//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

	if (Options.CuttingRegions == undefined) {
		// For optimal regions cutting we need cutting region to be exploded
		// But we do not explode it each time we need to cut region, we want to do that only once
		Options.CuttingRegions = new Array();
		ExplodeRegion(Options.CutRegionObject, Options.CuttingRegions);
	}
	
	CutRegion = new Array();
	// Loop through cutting regions (for case if cutting region consist of several non-intersecting regions)
	for (CuttingRegionNum = 0; CuttingRegionNum < Options.CuttingRegions.length; CuttingRegionNum++) {
		CuttingRegion = Options.CuttingRegions[CuttingRegionNum];
		// Loop through region's polygons
		for (PolygonNum = 0; PolygonNum < Region.length; PolygonNum++) {
			FirstPolygonNum = CutRegion.length;
			// Cut polygon by curent cutting region's outer polygon
			CutPolygonByPolygon(Region[PolygonNum], CuttingRegion[0], CutRegion, true);
			// Now cut each resulting polygon by cutting region's inner polygons (holes)
//			for (CuttingPolygonNum = 1; CuttingPolygonNum < ; CuttingPolygonNum++) {
				LastPolygonNum = CutRegion.length;
				// Loop through polygons created by cutting current region's polygon
				for (CutPolygonNum = 0; CutPolygonNum < LastPolygonNum - FirstPolygonNum; CutPolygonNum++) {
					CutPolygonByPolygon(CutRegion[FirstPolygonNum], CuttingRegion[CuttingPolygonNum], CutRegion, false);
					CutRegion.splice(FirstPolygonNum, 1);
				}
//			}
		}
	}
	return CutRegion;
}
//-------------------------------------------
function ProcessRegion(Region, LevelID, Options, CommonElements, SourceObjectLineNum) {
	var Regions, RegionsCnt, SplitDirection, CurRegionsCnt, Polygon;

	StatusMessage("Line " + SourceObjectLineNum + ", region at level " + LevelID + "...");
	
	ValidateRegion(Region, SourceObjectLineNum, LevelID);
	if (Options.CutRegionObject != undefined)
		Region = CutRegionByRegion(Region, Options.CutRegionObject[LevelID]);

	Regions = new Array();

	// Explode region to an array of non-intersecting regions
	StatusMessage("Line " + SourceObjectLineNum + ", region at level " + LevelID + ": Exploding...");
	ExplodeRegion(Region, Regions, SourceObjectLineNum, LevelID);
		
	// Split regions that are too large, convert to polygons and export them
	RegionsCnt = Regions.length;
	while (Regions.length > 0) {
		StatusMessage("Line " + SourceObjectLineNum + ", region at level " + LevelID + ": Splitting " + (RegionsCnt - Regions.length + 1) + "/" + RegionsCnt + "...");
		if ((SplitDirection = RegionRequiresSplitting(Regions[0], LevelID, Options)).SplitBy != "None") {
			CurRegionsCnt = Regions.length;
			// Split pregion
			SplitRegionTo(Regions[0], Regions, SplitDirection);
			Regions.shift();
			RegionsCnt += Regions.length - CurRegionsCnt;
			if (Regions.length == CurRegionsCnt) {
				ErrorMessage("Error: Region at line " + SourceObjectLineNum + ", level " + LevelID
				+ ": Region made of polygon" + (Regions[0][0].SourcePolygons.length == 1 ? " " : "s ") + Regions[0][0].SourcePolygons.join(", ")
				+ " was split to one region");
				// Convert region to polygon
				Polygon = RegionToPolygon(Regions[CurRegionsCnt - 1], SourceObjectLineNum, LevelID, RegionsCnt, RegionsCnt - Regions.length + 1);
				if (Polygon.Nodes.length > 2)
					// Export polygon
					ProcessDestinationPolygon(Polygon, LevelID, CommonElements, Options);
				Regions.pop();
			}
		} else {
			// Convert region to polygon
			Polygon = RegionToPolygon(Regions.shift(), SourceObjectLineNum, LevelID, RegionsCnt, RegionsCnt - Regions.length);
			if (Polygon.Nodes.length > 2)
				// Export polygon
				ProcessDestinationPolygon(Polygon, LevelID, CommonElements, Options);
		}
	}
}
//-------------------------------------------
function cmpLatitudeAscending(PrevNode, NextNode) {
	return PrevNode.Latitude - NextNode.Latitude;
}
//-------------------------------------------
function cmpLatitudeDescending(PrevNode, NextNode) {
	return NextNode.Latitude - PrevNode.Latitude;
}
//-------------------------------------------
function cmpLongitudeAscending(PrevNode, NextNode) {
	return PrevNode.Longitude - NextNode.Longitude;
}
//-------------------------------------------
function cmpLongitudeDescending(PrevNode, NextNode) {
	return NextNode.Longitude - PrevNode.Longitude;
}
//-------------------------------------------
function LinkAndRegionCrossingPoints(PrevNode, Node, CuttingRegion) {
	var CrossingPoints, PolygonID, Nodes, PolyNode, NodeNum, PolyPrevNode;
	var Denominator, Numerator1, Numerator2, Latitude, Longitude, PointNum, CrossingPoint;
	
	CrossingPoints = new Array();
	CrossingPoints.push(PrevNode);
	CrossingPoints.push(Node);
	for (PolygonID in CuttingRegion) {
		Polygon = CuttingRegion[PolygonID];
		if (PrevNode.Latitude < Polygon.MinLatitude && Node.Latitude < Polygon.MinLatitude
		|| PrevNode.Longitude < Polygon.MinLatitude && Node.Longitude < Polygon.MinLatitude
		|| PrevNode.Latitude > Polygon.MaxLatitude && Node.Latitude > Polygon.MaxLatitude
		|| PrevNode.Longitude > Polygon.MaxLatitude && Node.Longitude > Polygon.MaxLatitude
		|| Polygon.Nodes.length < 2)
			continue;
		Nodes = Polygon.Nodes;
		// Check all polygon's edges for crossing the link
		PolyNode = Polygon.Nodes[Polygon.Nodes.length - 1];
		for (NodeNum = 0; NodeNum < Polygon.Nodes.length; NodeNum++) {
			PolyPrevNode = PolyNode;
			PolyNode = Polygon.Nodes[NodeNum];
			
			if ((PrevNode.Latitude < PolyPrevNode.Latitude && PrevNode.Latitude < PolyNode.Latitude
			&& Node.Latitude < PolyPrevNode.Latitude && Node.Latitude < PolyNode.Latitude)
			|| (PrevNode.Latitude > PolyPrevNode.Latitude && PrevNode.Latitude > PolyNode.Latitude
			&& Node.Latitude > PolyPrevNode.Latitude && Node.Latitude > PolyNode.Latitude)
			|| (PrevNode.Longitude < PolyPrevNode.Longitude && PrevNode.Longitude < PolyNode.Longitude
			&& Node.Longitude < PolyPrevNode.Longitude && Node.Longitude < PolyNode.Longitude)
			|| (PrevNode.Longitude > PolyPrevNode.Longitude && PrevNode.Longitude > PolyNode.Longitude
			&& Node.Longitude > PolyPrevNode.Longitude && Node.Longitude > PolyNode.Longitude))
				continue; // Links are not crossing

			Denominator = (PolyNode.Longitude - PolyPrevNode.Longitude) * (Node.Latitude - PrevNode.Latitude)
			- (PolyNode.Latitude - PolyPrevNode.Latitude) * (Node.Longitude - PrevNode.Longitude);
			if (Denominator == 0) // Links are parallel or incident
				continue;

			Numerator1 = (PolyNode.Latitude - PolyPrevNode.Latitude) * (PrevNode.Longitude - PolyPrevNode.Longitude)
			- (PolyNode.Longitude - PolyPrevNode.Longitude) * (PrevNode.Latitude - PolyPrevNode.Latitude);
			Numerator2 = (Node.Latitude - PrevNode.Latitude) * (PrevNode.Longitude - PolyPrevNode.Longitude)
			- (Node.Longitude - PrevNode.Longitude) * (PrevNode.Latitude - PolyPrevNode.Latitude);
			
			if (Denominator > 0 && (Numerator1 < 0 || Numerator1 > Denominator || Numerator2 < 0 || Numerator2 > Denominator)
			|| Denominator < 0 && (Numerator1 > 0 || Numerator1 < Denominator || Numerator2 > 0 || Numerator2 < Denominator))
				continue; // Links are not crossing
			
			Latitude = PrevNode.Latitude + Numerator1 * (Node.Latitude - PrevNode.Latitude) / Denominator;
			Longitude = PrevNode.Longitude + Numerator1 * (Node.Longitude - PrevNode.Longitude) / Denominator;
			
			for (PointNum = 0; PointNum < CrossingPoints.length; PointNum++)
				if (CrossingPoints[PointNum].Latitude == Latitude && CrossingPoints[PointNum].Longitude == Longitude)
					break; // There is already a point in crossing points with the same coordinates
					
			if (PointNum == CrossingPoints.length) { // There was no such crossing point in the list
				CrossingPoint = new Object();
				CrossingPoint.Latitude = Latitude;
				CrossingPoint.Longitude = Longitude;
				CrossingPoints.push(CrossingPoint);
			}
		}
	}
	// Arrange crossing points in order from link start to end
	if (Math.abs(Node.Latitude - PrevNode.Latitude) > Math.abs(Node.Longitude - PrevNode.Longitude))
		if (Node.Latitude > PrevNode.Latitude)
			CrossingPoints.sort(cmpLatitudeAscending);
		else
			CrossingPoints.sort(cmpLatitudeDescending);
	else
		if (Node.Longitude > PrevNode.Longitude)
			CrossingPoints.sort(cmpLongitudeAscending);
		else
			CrossingPoints.sort(cmpLongitudeDescending);
			
	return CrossingPoints;
}
//-------------------------------------------
function CutPolylineByRegion(Polyline, CuttingRegion) {
	var CutPolyline, LineID, Line, Node, Nodes, CrossingPoints, CutLine;
	
	// Cut polyline by region
	CutPolyline = new Array();
	for (LineID in Polyline) {
		// Cut line by region
		Line = Polyline[LineID];
		// Process all links of source line
		Node = Line.Nodes[0];
		Nodes = new Array();
		for (NodeNum = 1; NodeNum < Line.Nodes.length; NodeNum++) {
			PrevNode = Node;
			Node = Line.Nodes[NodeNum];
			// Find all crossing points of current link with all polygons' borders of cutting region (arranged, start and end nodes included)
			CrossingPoints = LinkAndRegionCrossingPoints(PrevNode, Node, CuttingRegion);
			for (SubLinkNum = 0; SubLinkNum < CrossingPoints.length - 1; SubLinkNum++) {
				Middle = new Object();
				Middle.Latitude = (CrossingPoints[SubLinkNum].Latitude + CrossingPoints[SubLinkNum + 1].Latitude) / 2;
				Middle.Longitude = (CrossingPoints[SubLinkNum].Longitude + CrossingPoints[SubLinkNum + 1].Longitude) / 2;
				if (PointWithinRegion(Middle, CuttingRegion) >= 0) {
					// Middle of the link is within cutting region or at its border
					if (Nodes.length == 0) // First link of cut line
						Nodes.push(CrossingPoints[SubLinkNum]); // Push first node to cut line's nodes
					else if (SubLinkNum > 0) // Next consecutive sublink within current link
						Nodes.pop(); // Pop last node as it would be fake
					Nodes.push(CrossingPoints[SubLinkNum + 1]); // Push next node to cut line's nodes
				} else {
					// Middle of the link is not within cutting region or at its border
					if (Nodes.length > 0) { // We have already cut line
						CutLine = new Object();
						CutLine.Nodes = Nodes;
						SetSectionProperties(CutLine);
						CutPolyline.push(CutLine);
						Nodes = new Array();
					}
				}
			}
		}
		if (Nodes.length > 0) { // We have already cut line
			CutLine = new Object();
			CutLine.Nodes = Nodes;
			SetSectionProperties(CutLine);
			CutPolyline.push(CutLine);
		}
	}
	return CutPolyline;
}
//-------------------------------------------
function ProcessPolyline(Polyline, LevelID, Options, CommonElements, SourceObjectLineNum) {
	var MaximumSize, LinesCnt, LineNum, Line, Nodes, NodeNum, NewLine, NewNode;
	var MinLatitude, MinLongitude, MaxLatitude, MaxLongitude;

	StatusMessage("Line " + SourceObjectLineNum + ", polyline at level " + LevelID + "...");

	ValidatePolyline(Polyline, SourceObjectLineNum, LevelID);
	if (Options.CutRegionObject != undefined)
		Polyline = CutPolylineByRegion(Polyline, Options.CutRegionObject[LevelID]);
	
	if (Options.SplitMode > 0) {
		// Split each Line by map blocks
		LinesCnt = Polyline.length;
		for (LineNum = 0; LineNum < LinesCnt; LineNum++) {
			StatusMessage("Line " + SourceObjectLineNum + ", polyline at level " + LevelID + ": Splitting lines by map blocks " + (LineNum + 1 ) + "/" + LinesCnt + "...");
			Line = Polyline[LineNum];
			Nodes = Line.Nodes;
			for (NodeNum = 0; NodeNum < Nodes.length; NodeNum++) {
				if (NodeNum == 0) {
					if (Options.SplitMode == 1) {
						MinLatitude = Math.floor(Nodes[0].Latitude / 2 + o) * 2;
						MaxLatitude = Math.ceil(Nodes[0].Latitude / 2 - o) * 2;
						MinLongitude = Math.floor(Nodes[0].Longitude / 2 + o) * 2;
						MaxLongitude = Math.ceil(Nodes[0].Longitude / 2 - o) * 2;
					} else if (Options.SplitMode > 1) {
						SplitScale = Options.SplitMode == 2 ? 1 : Options.SplitMode == 3 ? 2 : Options.SplitMode == 4 ? 6 : 12;
						MinLatitude = Math.floor(Nodes[0].Latitude * SplitScale / 4 + o) * 4 / SplitScale;
						MaxLatitude = Math.ceil(Nodes[0].Latitude * SplitScale / 4 - o) * 4 / SplitScale;
						if (MaxLatitude <= 60 && MinLatitude >= -60) {
							MinLongitude = Math.floor((Nodes[0].Longitude + 180) * SplitScale / 6 + o) * 6 / SplitScale - 180;
							MaxLongitude = Math.ceil((Nodes[0].Longitude + 180) * SplitScale / 6 - o) * 6 / SplitScale - 180;
						} else if (MaxLatitude <= 76 && MinLatitude >= -76) {
							MinLongitude = Math.floor((Nodes[0].Longitude + 180) * SplitScale / 12 + o) * 12 / SplitScale - 180;
							MaxLongitude = Math.ceil((Nodes[0].Longitude + 180) * SplitScale / 12 - o) * 12 / SplitScale - 180;
						} else {
							MinLongitude = Math.floor((Nodes[0].Longitude + 180) * SplitScale / 24 + o) * 24 / SplitScale - 180;
							MaxLongitude = Math.ceil((Nodes[0].Longitude + 180) * SplitScale / 24 - o) * 24 / SplitScale - 180;
						}
					}
				} else { // if (NodeNum > 0)
					// Not a first node
					
					// If block by latitude is still not defined
					if (MinLatitude == MaxLatitude && Nodes[NodeNum].Latitude != MinLatitude)
						if (Options.SplitMode == 1) {
							if (Nodes[NodeNum].Latitude > MinLatitude)
								MaxLatitude = MinLatitude + 2;
							else
								MinLatitude = MaxLatitude - 2;
						} else { // if (Options.SplitMode > 1)
							if (Nodes[NodeNum].Latitude > MinLatitude)
								MaxLatitude = MinLatitude + 4 / SplitScale;
							else
								MinLatitude = MaxLatitude - 4 / SplitScale;
						}
				
					// If block by longitude is still not defined
					if (MinLongitude == MaxLongitude && Nodes[NodeNum].Longitude != MinLongitude)
						if (Options.SplitMode == 1) {
							if (Nodes[NodeNum].Longitude > MinLongitude)
								MaxLongitude = MinLongitude + 2;
							else
								MinLongitude = MaxLongitude - 2;
						} else { // if (Options.SplitMode > 1)
							if (Nodes[NodeNum].Longitude > MinLongitude)
								if (MaxLatitude <= 60 && MinLatitude >= -60) {
									MaxLongitude = MinLongitude + 6 / SplitScale;
								} else if (MaxLatitude <= 76 && MinLatitude >= -76) {
									MaxLongitude = MinLongitude + 12 / SplitScale;
								} else {
									MaxLongitude = MinLongitude + 24 / SplitScale;
								}
							else
								if (MaxLatitude <= 60 && MinLatitude >= -60) {
									MinLongitude = MaxLongitude - 6 / SplitScale;
								} else if (MaxLatitude <= 76 && MinLatitude >= -76) {
									MinLongitude = MaxLongitude - 12 / SplitScale;
								} else {
									MaxLongitude = MinLongitude - 24 / SplitScale;
								}
						}
						
					// Check current link for going outside current block
					PrevLatitude = Nodes[NodeNum - 1].Latitude;
					PrevLongitude = Nodes[NodeNum - 1].Longitude;
					Latitude = Nodes[NodeNum].Latitude;
					Longitude = Nodes[NodeNum].Longitude;

					if (Latitude > MaxLatitude) {
						Longitude = Longitude + (PrevLongitude - Longitude) * (MaxLatitude - Latitude) / (PrevLatitude - Latitude); 
						Latitude = MaxLatitude;
					} else if(Latitude < MinLatitude) {
						Longitude = Longitude + (PrevLongitude - Longitude) * (MinLatitude - Latitude) / (PrevLatitude - Latitude); 
						Latitude = MinLatitude;
					}
				
					if (Longitude > MaxLongitude) {
						Latitude = Nodes[NodeNum].Latitude + (PrevLatitude - Nodes[NodeNum].Latitude)
						* (MaxLongitude - Nodes[NodeNum].Longitude) / (PrevLongitude - Nodes[NodeNum].Longitude); 
						Longitude = MaxLongitude;
					} else if(Longitude < MinLongitude) {
						Latitude = Nodes[NodeNum].Latitude + (PrevLatitude - Nodes[NodeNum].Latitude)
						* (MinLongitude - Nodes[NodeNum].Longitude) / (PrevLongitude - Nodes[NodeNum].Longitude); 
						Longitude = MinLongitude;
					}
				
					if (Latitude == Nodes[NodeNum - 1].Latitude && Longitude == Nodes[NodeNum - 1].Longitude) {
						// Current link is entirely outside current map block
						NewLine = new Object();
						NewLine.Nodes = Nodes.slice(0, NodeNum);
						NewLine.SourceSections = new Array();
						NewLine.SourceSections = Line.SourceSections.concat(Line.SourceSections);

						Polyline.push(NewLine);
						Nodes.splice(0, NodeNum - 1);
						NodeNum = -1;
					} else if (Latitude != Nodes[NodeNum].Latitude || Longitude != Nodes[NodeNum].Longitude) {
						// Current link is partially within current map block
						NewNode = new Object();
						NewNode.Latitude = Latitude;
						NewNode.Longitude = Longitude;
						Nodes.splice(NodeNum, 0, NewNode);

						NewLine = new Object();
						NewLine.Nodes = Nodes.slice(0, NodeNum + 1);
						NewLine.SourceSections = Line.SourceSections.concat();

						Polyline.push(NewLine);
						Nodes.splice(0, NodeNum);
						NodeNum = -1;
					}
				}
			}
		}
	}

	if (ImgParams["tresize"] != undefined && LevelGrid[LevelID] != undefined)
		MaximumSize = ImgParams["tresize"] * LevelGrid[LevelID];
	else
		MaximumSize = 360;
		
	LinesCnt = Polyline.length;
	while (Polyline.length > 0) {
		StatusMessage("Line " + SourceObjectLineNum + ", polyline at level " + LevelID + ": Processing lines " + (LinesCnt - Polyline.length + 1 ) + "/" + LinesCnt + "...");
		Line = Polyline.shift();
		Nodes = Line.Nodes;
		for (NodeNum = 0; NodeNum < Nodes.length; NodeNum++) {
			if (NodeNum == 0) {

				MinLatitude = Nodes[0].Latitude;
				MinLongitude = Nodes[0].Longitude;
				MaxLatitude = Nodes[0].Latitude;
				MaxLongitude = Nodes[0].Longitude;
				
			} else if (Options.SplitPolylines && NodeNum == MaxNodesCnt) {

				NewLine = new Object();
				NewLine.Nodes = Nodes.slice(0, NodeNum);
				NewLine.MinLatitude = MinLatitude;
				NewLine.MinLongitude = MinLongitude;
				NewLine.MaxLatitude = MaxLatitude;
				NewLine.MaxLongitude = MaxLongitude;
				NewLine.SourceSections = new Array();
				NewLine.SourceSections = Line.SourceSections.concat(Line.SourceSections);

				ProcessDestinationLine(NewLine, LevelID, CommonElements, Options)
				Nodes.splice(0, NodeNum - 1);
				NodeNum = -1;
			
			} else if (Options.SplitPolylines && Math.max(MaxLatitude, Nodes[NodeNum].Latitude) - Math.min(MinLatitude, Nodes[NodeNum].Latitude)
			>= MaximumSize
			|| Math.max(MaxLongitude, Nodes[NodeNum].Longitude) - Math.min(MinLongitude, Nodes[NodeNum].Longitude)
			>= MaximumSize) {
				if (NodeNum == 1) {
					// We've got a line of only two nodes, which exceeds maximum TRE size
					// We should add one more node
					if (Math.abs(Nodes[1].Latitude - Nodes[0].Latitude) < MaximumSize
					|| Math.abs(Nodes[1].Latitude - Nodes[0].Latitude) >= MaximumSize
					&& Math.abs(Nodes[1].Latitude - Nodes[0].Latitude) < Math.abs(Nodes[1].Longitude - Nodes[0].Longitude)) {
						// Split by longitude
						Longitude = Nodes[1].Longitude > Nodes[0].Longitude
						? Nodes[0].Longitude + MaximumSize : Nodes[0].Longitude - MaximumSize;
						Latitude = Nodes[1].Latitude == Nodes[0].Latitude ? Nodes[0].Latitude
						: Nodes[0].Latitude + (Nodes[1].Latitude - Nodes[0].Latitude)
						/ (Nodes[1].Longitude - Nodes[0].Longitude) * (Longitude - Nodes[0].Longitude);
					} else if (Math.abs(Nodes[1].Longitude - Nodes[0].Longitude) < MaximumSize
					|| Math.abs(Nodes[1].Longitude - Nodes[0].Longitude) >= MaximumSize
					&& Math.abs(Nodes[1].Longitude - Nodes[0].Longitude) < Math.abs(Nodes[1].Latitude - Nodes[0].Latitude)) {
						// Split by latitude
						Latitude = Nodes[1].Latitude > Nodes[0].Latitude
						? Nodes[0].Latitude + MaximumSize : Nodes[0].Latitude - MaximumSize;
						Longitude = Nodes[1].Longitude == Nodes[0].Longitude ? Nodes[0].Longitude
						: Nodes[0].Longitude + (Nodes[1].Longitude - Nodes[0].Longitude)
						/ (Nodes[1].Latitude - Nodes[0].Latitude) * (Latitude - Nodes[0].Latitude);
					}
					NewNode = new Object();
					NewNode.Latitude = Latitude;
					NewNode.Longitude = Longitude;
					Nodes.splice(1, 0, NewNode);
					MinLatitude = Math.min(MinLatitude, Latitude);
					MinLongitude = Math.min(MinLongitude, Longitude);
					MaxLatitude = Math.max(MaxLatitude, Latitude);
					MaxLongitude = Math.max(MaxLongitude, Longitude);
					NodeNum++;
				}
				NewLine = new Object();
				NewLine.Nodes = Nodes.slice(0, NodeNum);
				NewLine.MinLatitude = MinLatitude;
				NewLine.MinLongitude = MinLongitude;
				NewLine.MaxLatitude = MaxLatitude;
				NewLine.MaxLongitude = MaxLongitude;
				NewLine.SourceSections = Line.SourceSections.concat();

				ProcessDestinationLine(NewLine, LevelID, CommonElements, Options)
				Nodes.splice(0, NodeNum - 1);
				NodeNum = -1;
			} else {
				MinLatitude = Math.min(MinLatitude, Nodes[NodeNum].Latitude);
				MinLongitude = Math.min(MinLongitude, Nodes[NodeNum].Longitude);
				MaxLatitude = Math.max(MaxLatitude, Nodes[NodeNum].Latitude);
				MaxLongitude = Math.max(MaxLongitude, Nodes[NodeNum].Longitude);
			}
		}
		Line.MinLatitude = MinLatitude;
		Line.MinLongitude = MinLongitude;
		Line.MaxLatitude = MaxLatitude;
		Line.MaxLongitude = MaxLongitude;
		ProcessDestinationLine(Line, LevelID, CommonElements, Options)
	}
}
//-------------------------------------------
function SetPolygonProperties(Polygon) {
	// Sets polygon Area, Direction, MinLatitude, MinLongitude, MaxLatitude, MaxLongitude
	var NodeNum;
	
	for (NodeNum = 0; NodeNum < Polygon.Nodes.length; NodeNum++)
		if (NodeNum == 0) {

			Polygon.MinLatitude = Polygon.Nodes[NodeNum].Latitude;
			Polygon.MinLongitude = Polygon.Nodes[NodeNum].Longitude;
			Polygon.MaxLatitude = Polygon.Nodes[NodeNum].Latitude;
			Polygon.MaxLongitude = Polygon.Nodes[NodeNum].Longitude;

			if (NodeNum < Polygon.Nodes.length - 1)
				Polygon.Area = (Polygon.Nodes[NodeNum + 1].Latitude - Polygon.Nodes[NodeNum].Latitude)
				* (Polygon.Nodes[NodeNum + 1].Longitude + Polygon.Nodes[NodeNum].Longitude)
			else
				Polygon.Area = 0;

		} else {

			Polygon.MinLatitude = Math.min(Polygon.MinLatitude, Polygon.Nodes[NodeNum].Latitude);
			Polygon.MinLongitude = Math.min(Polygon.MinLongitude, Polygon.Nodes[NodeNum].Longitude);
			Polygon.MaxLatitude = Math.max(Polygon.MaxLatitude, Polygon.Nodes[NodeNum].Latitude);
			Polygon.MaxLongitude = Math.max(Polygon.MaxLongitude, Polygon.Nodes[NodeNum].Longitude);

			if (NodeNum < Polygon.Nodes.length - 1)
				Polygon.Area += (Polygon.Nodes[NodeNum + 1].Latitude - Polygon.Nodes[NodeNum].Latitude)
				* (Polygon.Nodes[NodeNum + 1].Longitude + Polygon.Nodes[NodeNum].Longitude)
			else
				Polygon.Area += (Polygon.Nodes[0].Latitude - Polygon.Nodes[NodeNum].Latitude)
				* (Polygon.Nodes[0].Longitude + Polygon.Nodes[NodeNum].Longitude)

		}
		
	Polygon.Direction = Polygon.Area > 0 ? 1 : Polygon.Area < 0 ? -1 : 0;
	Polygon.Area = Polygon.Area * Polygon.Direction / 2;
	
}
//-------------------------------------------
function SetSectionProperties(Section) {
	// Sets section MinLatitude, MinLongitude, MaxLatitude, MaxLongitude
	var NodeNum;
	
	for (NodeNum = 0; NodeNum < Section.Nodes.length; NodeNum++)
		if (NodeNum == 0) {
			Section.MinLatitude = Section.Nodes[NodeNum].Latitude;
			Section.MinLongitude = Section.Nodes[NodeNum].Longitude;
			Section.MaxLatitude = Section.Nodes[NodeNum].Latitude;
			Section.MaxLongitude = Section.Nodes[NodeNum].Longitude;
		} else {
			Section.MinLatitude = Math.min(Section.MinLatitude, Section.Nodes[NodeNum].Latitude);
			Section.MinLongitude = Math.min(Section.MinLongitude, Section.Nodes[NodeNum].Longitude);
			Section.MaxLatitude = Math.max(Section.MaxLatitude, Section.Nodes[NodeNum].Latitude);
			Section.MaxLongitude = Math.max(Section.MaxLongitude, Section.Nodes[NodeNum].Longitude);
		}
}
//-------------------------------------------
function ValidatePolygon(Polygon) {
	var Nodes, NodeNum;

	Nodes = Polygon.Nodes;
	// Delete fake nodes between first and last nodes
	for (NodeNum = 1; NodeNum < Nodes.length; NodeNum++) {
		if (Nodes[NodeNum - 1].Latitude == Nodes[NodeNum].Latitude
		&& Nodes[NodeNum - 1].Longitude == Nodes[NodeNum].Longitude) {
			Nodes.splice(NodeNum, 1);
			NodeNum--;
		} else if(NodeNum > 1
		&& ((Nodes[NodeNum - 2].Latitude == Nodes[NodeNum - 1].Latitude
		&& Nodes[NodeNum].Latitude == Nodes[NodeNum - 1].Latitude)
		|| (Nodes[NodeNum - 2].Latitude != Nodes[NodeNum - 1].Latitude
		&& Nodes[NodeNum].Longitude == Nodes[NodeNum - 1].Longitude
		+ (Nodes[NodeNum - 2].Longitude - Nodes[NodeNum - 1].Longitude)
		/ (Nodes[NodeNum - 2].Latitude - Nodes[NodeNum - 1].Latitude)
		* (Nodes[NodeNum].Latitude - Nodes[NodeNum - 1].Latitude)))) {
			Nodes.splice(NodeNum - 1, 1);
			NodeNum -= 2;
		}
	}
	// Delete fake nodes at begin and at the end of polygon
	while (Nodes.length > 1) {
		// Check whether last node is equal to first one
		if (Nodes[Nodes.length - 1].Latitude == Nodes[0].Latitude
		&& Nodes[Nodes.length - 1].Longitude == Nodes[0].Longitude) {
			Nodes.pop();
			continue;
		}
		// Check whether last node is fake
		if (Nodes.length > 2
		&& ((Nodes[Nodes.length - 2].Latitude == Nodes[Nodes.length - 1].Latitude
		&& Nodes[0].Latitude == Nodes[Nodes.length - 1].Latitude)
		|| (Nodes[Nodes.length - 2].Latitude != Nodes[Nodes.length - 1].Latitude
		&& Nodes[0].Longitude == Nodes[Nodes.length - 1].Longitude
		+ (Nodes[Nodes.length - 2].Longitude - Nodes[Nodes.length - 1].Longitude)
		/ (Nodes[Nodes.length - 2].Latitude - Nodes[Nodes.length - 1].Latitude)
		* (Nodes[0].Latitude - Nodes[Nodes.length - 1].Latitude)))) {
			Nodes.pop();
			continue;
		}
		// Check whether first node is fake
		if (Nodes.length > 2
		&& ((Nodes[Nodes.length - 1].Latitude == Nodes[0].Latitude
		&& Nodes[1].Latitude == Nodes[0].Latitude)
		|| (Nodes[Nodes.length - 1].Latitude != Nodes[0].Latitude
		&& Nodes[1].Longitude == Nodes[0].Longitude
		+ (Nodes[Nodes.length - 1].Longitude - Nodes[0].Longitude)
		/ (Nodes[Nodes.length - 1].Latitude - Nodes[0].Latitude)
		* (Nodes[1].Latitude - Nodes[0].Latitude)))) {
			Nodes.shift();
			continue;
		}
		break;
	}
	SetPolygonProperties(Polygon);
	if (Polygon.Direction < 0) {
		Polygon.Nodes.reverse();
		Polygon.Direction = -Polygon.Direction;
	}
}
//-------------------------------------------
function ImportNode(NodeSource) {
	// Imports and returns node from text string
	var Node, SeparatorIndex, Latitude, Longitude;

	NodeSource = LTrim(NodeSource);
	if (NodeSource.charAt(0) == "(") {
		NodeSource = LTrim(NodeSource.substring(1, NodeSource.length)); // remove leading "("

		// Get latitude
		SeparatorIndex = NodeSource.indexOf(",");
		Latitude = parseFloat(RTrim(NodeSource.substring(0, SeparatorIndex)));
		NodeSource = LTrim(NodeSource.substring(SeparatorIndex + 1, NodeSource.length)); // remove latitude value and comma

		// Get longitude
		SeparatorIndex = NodeSource.indexOf(")");
		Longitude = parseFloat(RTrim(NodeSource.substring(0, SeparatorIndex)));
		
		Node = new Object();
		if (ImgParams["datum"] != undefined && ImgParams["datum"].toUpperCase() == "SK-42") {
			Node.Latitude = WGS84Lat(Latitude, Longitude, 0);
			Node.Longitude = WGS84Long(Latitude, Longitude, 0);
		} else {
			Node.Latitude = Latitude;
			Node.Longitude = Longitude;
		}
		NodesProcessed++;
	}
	
	return Node;
}
//-------------------------------------------
function ImportPolygon(PolygonSource) {
	// Imports and returns polygon from text string
	var CharPos, CurChar, Polygon, Nodes, SeparatorIndex, Latitude, Longitude, Node;

	CharPos = 0;
	while (CurChar = PolygonSource.charAt(CharPos), CurChar == " " || CurChar == "\t") CharPos++;
	Polygon = new Object();
	Nodes = new Array();
	while (PolygonSource.charAt(CharPos) == "(") {
		CharPos++;
		while (CurChar = PolygonSource.charAt(CharPos), CurChar == " " || CurChar == "\t") CharPos++;

		// Get latitude
		if((SeparatorIndex = PolygonSource.indexOf(",", CharPos)) < 0) break;
		Latitude = parseFloat(RTrim(PolygonSource.substring(CharPos, SeparatorIndex)));
		CharPos = SeparatorIndex + 1; // pass latitude value and comma
		while (CurChar = PolygonSource.charAt(CharPos), CurChar == " " || CurChar == "\t") CharPos++;

		// Get longitude
		if((SeparatorIndex = PolygonSource.indexOf(")", CharPos)) < 0) break;
		Longitude = parseFloat(RTrim(PolygonSource.substring(CharPos, SeparatorIndex)));
		CharPos = SeparatorIndex + 1; // pass longitude value and closing bracket
		
		Node = new Object();
		if (ImgParams["datum"] != undefined && ImgParams["datum"].toUpperCase() == "SK-42") {
			Node.Latitude = WGS84Lat(Latitude, Longitude, 0);
			Node.Longitude = WGS84Long(Latitude, Longitude, 0);
		} else {
			Node.Latitude = Latitude;
			Node.Longitude = Longitude;
		}
		Nodes.push(Node);
		NodesProcessed++;

		if((SeparatorIndex = PolygonSource.indexOf(",", CharPos)) < 0) break;
		CharPos = SeparatorIndex + 1; // pass comma
		while (CurChar = PolygonSource.charAt(CharPos), CurChar == " " || CurChar == "\t") CharPos++;
	}
	Polygon.Nodes = Nodes;
	
	return Polygon;
}
//-------------------------------------------
function ImportLine(SectionSource) {
	return ImportPolygon(SectionSource);
}
//-------------------------------------------
function ProcessMapSection(SourceSection, SourceObjectLineNum, Destination, Options) {
	var ElementNum, EquationSignIndex, ElementID, ElementValue, LevelID;
	
	if (typeof(Destination) != "string") {
		ErrorMessage("Error: Section type [Map] not supported in cutting file. Result may be unpredictable.");
		return;
	}

	if (Options == undefined || Options.SplitMode == 0) {
		ErrorMessage("Error: Section type [Map] (line " + SourceObjectLineNum + ") not supported in non-splitting mode. Section removed.");
		return;
	}

	for (ElementNum = 1; ElementNum < SourceSection.length - 1; ElementNum++) {
		EquationSignIndex = SourceSection[ElementNum].indexOf("=");
		if (EquationSignIndex != -1) {
			ElementID = Trim(SourceSection[ElementNum].substr(0, EquationSignIndex)).toLowerCase();
			ElementValue = Trim(SourceSection[ElementNum].substring(EquationSignIndex + 1, SourceSection[ElementNum].length));
			MapParams[ElementID] = ElementValue;
		}
	}

	MapSection = SourceSection;
}
//-------------------------------------------
function ProcessImgSection(SourceSection, SourceObjectLineNum, Destination, Options) {
	var ElementNum, EquationSignIndex, ElementID, ElementValue, LevelID;

	if (typeof(Destination) != "string") {
		ErrorMessage("Error: Section type [Map] not supported in cutting file. Result may be unpredictable.");
		return;
	}

	for (ElementNum = 1; ElementNum < SourceSection.length - 1; ElementNum++) {
		EquationSignIndex = SourceSection[ElementNum].indexOf("=");
		if (EquationSignIndex != -1) {
			ElementID = Trim(SourceSection[ElementNum].substr(0, EquationSignIndex)).toLowerCase();
			ElementValue = LTrim(SourceSection[ElementNum].substring(EquationSignIndex + 1, SourceSection[ElementNum].length));
			ImgParams[ElementID] = ElementValue;
		}
	}
	if (ImgParams["levels"] == undefined)
		ErrorMessage("Error: Levels value in [IMG ID] section undefined! Regions and lines will be split disregarding of maximum TRE size for each level.");
	else
		for (LevelID = 0; LevelID < ImgParams["levels"]; LevelID++)
			if (ImgParams["level" + LevelID] == undefined)
				ErrorMessage("Error: Level" + LevelID + " value in [IMG ID] section undefined");
			else
				LevelGrid[LevelID] = 360 / Math.pow(2, ImgParams["level" + LevelID]);
				
	if (Options == undefined || Options.SplitMode == 0)
		ProcessDestinationSection(SourceSection);
	else
		ImgSection = SourceSection;
}
//-------------------------------------------
function ProcessPOISection(SourceSection, SourceObjectLineNum, Destination, Options) {
	var MinLatitude, MinLongitude, MaxLatitude, MaxLongitude, MinLevelID;
	var ElementNum, EquationSignIndex, ElementID, ElementValue, LevelID, Node;

	if (typeof(Destination) != "string") {
		ErrorMessage("Error: Section type [RGN10] not supported in cutting file. Result may be unpredictable.");
		return;
	}

	StatusMessage("Line " + SourceObjectLineNum + ": Examining block...");
	for (ElementNum = 1; ElementNum < SourceSection.length - 1; ElementNum++) {
		EquationSignIndex = SourceSection[ElementNum].indexOf("=");
		if (EquationSignIndex != -1) {
			ElementID = Trim(SourceSection[ElementNum].substr(0, EquationSignIndex));
			ElementValue = Trim(SourceSection[ElementNum].substring(EquationSignIndex + 1, SourceSection[ElementNum].length));
			if (ElementID.substr(0, 4).toUpperCase() == "DATA"
			|| ElementID.substr(0, 6).toUpperCase() == "ORIGIN") {
				LevelID = parseInt(ElementID.substring((ElementID.substr(0, 6).toUpperCase() == "ORIGIN" ? 6 : 4), ElementID.length));
				Node = ImportNode(ElementValue);
				if (Node != undefined) {
					if (MinLevelID == undefined || LevelID < MinLevelID) {
						MinLatitude = Node.Latitude;
						MinLongitude = Node.Longitude;
						MaxLatitude = Node.Latitude;
						MaxLongitude = Node.Longitude;
						MinLevelID = LevelID;
					}
					SourceSection[ElementNum] = ElementID + "=" + ExportNode(Node);
				}
			}
		}
	}
	if (MinLevelID != undefined) {
		if (Options.CutRegionObject != undefined) {
			if (PointWithinRegion(Node, Options.CutRegionObject[MinLevelID]) == -1)
				return;
		}
		ProcessDestinationSection(SourceSection, MinLatitude, MinLongitude, MaxLatitude, MaxLongitude, Options);
		POIsProcessed++;
	} else
		ErrorMessage("Error: POI at line " + SourceObjectLineNum + " is fake");
}
//-------------------------------------------
function ProcessCityPointSection(SourceSection, SourceObjectLineNum, Destination, Options) {
	var MinLatitude, MinLongitude, MaxLatitude, MaxLongitude, MinLevelID;
	var ElementNum, EquationSignIndex, ElementID, ElementValue, LevelID, Node;

	if (typeof(Destination) != "string") {
		ErrorMessage("Error: Section type [RGN20] not supported in cutting file. Result may be unpredictable.");
		return;
	}

	StatusMessage("Line " + SourceObjectLineNum + ": Examining block...");
	for (ElementNum = 1; ElementNum < SourceSection.length - 1; ElementNum++) {
		EquationSignIndex = SourceSection[ElementNum].indexOf("=");
		if (EquationSignIndex != -1) {
			ElementID = Trim(SourceSection[ElementNum].substr(0, EquationSignIndex));
			ElementValue = Trim(SourceSection[ElementNum].substring(EquationSignIndex + 1, SourceSection[ElementNum].length));
			if (ElementID.substr(0, 4).toUpperCase() == "DATA"
			|| ElementID.substr(0, 6).toUpperCase() == "ORIGIN") {
				LevelID = parseInt(ElementID.substring((ElementID.substr(0, 6).toUpperCase() == "ORIGIN" ? 6 : 4), ElementID.length));
				Node = ImportNode(ElementValue);
				if (Node != undefined) {
					if (MinLevelID == undefined || LevelID < MinLevelID) {
						MinLatitude = Node.Latitude;
						MinLongitude = Node.Longitude;
						MaxLatitude = Node.Latitude;
						MaxLongitude = Node.Longitude;
						MinLevelID = LevelID;
					}
					SourceSection[ElementNum] = ElementID + "=" + ExportNode(Node);
				}
			}
		}
	}
	if (MinLevelID != undefined) {
		if (Options.CutRegionObject != undefined) {
			if (PointWithinRegion(Node, Options.CutRegionObject[MinLevelID]) == -1)
				return;
		}
		ProcessDestinationSection(SourceSection, MinLatitude, MinLongitude, MaxLatitude, MaxLongitude, Options);
		CitiesPointsProcessed++;
	} else
		ErrorMessage("Error: City/Point at line " + SourceObjectLineNum + " is fake");
}
//-------------------------------------------
function ProcessPolylineSection(SourceSection, SourceObjectLineNum, Destination, Options) {
	var CommonElements, Levels, ElementNum, EquationSignIndex, ElementID, ElementValue, LevelID, Line;

	if (typeof(Destination) != "string") {
		ErrorMessage("Error: Section type [RGN40] not supported in cutting file. Result may be unpredictable.");
		return;
	}

	CommonElements = new Array();
	Levels = new Array();
	for (ElementNum = 1; ElementNum < SourceSection.length - 1; ElementNum++) {
		StatusMessage("Line " + SourceObjectLineNum + ": Importing block (" + (ElementNum + 1) + "/" + SourceSection.length + ")...");
		EquationSignIndex = SourceSection[ElementNum].indexOf("=");
		if (EquationSignIndex != -1) {
			ElementID = Trim(SourceSection[ElementNum].substr(0, EquationSignIndex));
			ElementValue = Trim(SourceSection[ElementNum].substring(EquationSignIndex + 1, SourceSection[ElementNum].length));
			if (ElementID.substr(0, 4).toUpperCase() != "DATA")
				CommonElements.push(SourceSection[ElementNum]);
			else {
				LevelID = parseInt(ElementID.substring(4, ElementID.length));
				if (Levels[LevelID] == undefined)
					Levels[LevelID] = new Array();
				Line = ImportLine(ElementValue);
				Line.SourceSections = new Array();
				Line.SourceSections.push(Levels[LevelID].length + 1);
				Levels[LevelID].push(Line);
			}
		}
	}
	// Process polylines for each layer
	for (LevelID in Levels) {
		ProcessPolyline(Levels[LevelID], LevelID, Options, CommonElements, SourceObjectLineNum);
		PolylinesProcessed++;
	}
}
//-------------------------------------------
function ProcessRegionSection(SourceSection, SourceObjectLineNum, Destination, Options) {
	var CommonElements, Levels, ElementNum, EquationSignIndex, ElementID, ElementValue, LevelID, Polygon;

	CommonElements = new Array();
	if (typeof(Destination) == "string")
		Levels = new Array();
	else
		Levels = Destination;
	for (ElementNum = 1; ElementNum < SourceSection.length - 1; ElementNum++) {
		StatusMessage("Line " + SourceObjectLineNum + ": Importing block (" + (ElementNum + 1) + "/" + SourceSection.length + ")...");
		EquationSignIndex = SourceSection[ElementNum].indexOf("=");
		if (EquationSignIndex != -1) {
			ElementID = Trim(SourceSection[ElementNum].substr(0, EquationSignIndex));
			ElementValue = Trim(SourceSection[ElementNum].substring(EquationSignIndex + 1, SourceSection[ElementNum].length));
			if (ElementID.substr(0, 4).toUpperCase() != "DATA")
				CommonElements.push(SourceSection[ElementNum]);
			else {
				LevelID = parseInt(ElementID.substring(4, ElementID.length));
				if (Levels[LevelID] == undefined)
					Levels[LevelID] = new Array();
				Polygon = ImportPolygon(ElementValue);
				Polygon.SourcePolygons = new Array();
				Polygon.SourcePolygons.push(Levels[LevelID].length + 1);
				Levels[LevelID].push(Polygon);
			}
		}
	}
	if (typeof(Destination) == "string")
		// Process regions for each layer
		for (LevelID in Levels) {
			ProcessRegion(Levels[LevelID], LevelID, Options, CommonElements, SourceObjectLineNum);
			RegionsProcessed++;
		}
}
//-------------------------------------------
function ProcessSection(SourceSection, SourceObjectLineNum, Destination, Options) {

	if (SourceSection[0].substr(0, 8).toUpperCase() == "[MAP]")
		// Section represents map block header
		ProcessMapSection(SourceSection, SourceObjectLineNum, Destination, Options);
	else if (SourceSection[0].substr(0, 8).toUpperCase() == "[IMG ID]")
		// Section represents map block header
		ProcessImgSection(SourceSection, SourceObjectLineNum, Destination, Options);
	else if (SourceSection[0].substr(0, 7).toUpperCase() == "[RGN10]")
		// Section represents POI
		ProcessPOISection(SourceSection, SourceObjectLineNum, Destination, Options);
	else if (SourceSection[0].substr(0, 7).toUpperCase() == "[RGN20]")
		// Section represents City or Point
		ProcessCityPointSection(SourceSection, SourceObjectLineNum, Destination, Options);
	else if (SourceSection[0].substr(0, 7).toUpperCase() == "[RGN40]")
		// Section represents polyline
		ProcessPolylineSection(SourceSection, SourceObjectLineNum, Destination, Options);
	else if (SourceSection[0].substr(0, 7).toUpperCase() == "[RGN80]")
		// Section represents region
		ProcessRegionSection(SourceSection, SourceObjectLineNum, Destination, Options);
	else
		ErrorMessage("Error: Section type " + SourceSection[0] + " (line " + SourceObjectLineNum + ") not supported. Section removed.");
}
//-------------------------------------------
function ProcessFile(SourceFileName, Destination, Options) {

	SourceFile = fso.OpenTextFile(SourceFileName, ForReading);

	BlockStarted = false;
	while (!SourceFile.AtEndOfStream) {

		SourceLine = SourceFile.ReadLine();
		if (!BlockStarted) {
			StatusMessage("Line " + SourceFile.Line + "...");
			if (SourceLine.charAt(0) == "[") {
				SourceBlock = new Array();
				SourceBlock.push(SourceLine);
				SourceObjectLineNum = SourceFile.Line - 1;
				BlockStarted = true;
			} else if ((Options == undefined || Options.SplitMode == 0) && typeof(Destination) == "string")
				ProcessDestinationString(SourceLine);
		} else {
			StatusMessage("Line " + SourceObjectLineNum + ": Reading block (" + SourceBlock.length + ")...");
			SourceBlock.push(SourceLine);
			if (SourceLine.charAt(0) == "[") {
				ProcessSection(SourceBlock, SourceObjectLineNum, Destination, Options);
				BlockStarted = false;
				SectionsProcessed++;
			}
		}
	}
	
	SourceFile.Close();
}
//-------------------------------------------
function ImportNames(Options) {
	var NamesFile, Name;

	StatusMessage("Reading block names...");
	var NamesFile = fso.OpenTextFile(Options.NamesFileName, ForReading, false, TristateFalse);
	while (!NamesFile.AtEndOfStream) {
		Name = NamesFile.ReadLine().split("\t");
		if (Name.length == 2)
			Options.Names[Name[0]] = Name[1];
	}
	NamesFile.Close();
}
//-------------------------------------------
function ProcessFiles(SourceFileName, DestinationFileOrFolderName, Options) {
	var StartDate, EndDate, TimeElapsed;
	var SourceFile, SourceLine, BlockStarted, SourceBlock, DestinationBlock;
	var LineNum, SourceObjectLineNum;
	
	SectionsProcessed = 0;
	NodesProcessed = 0;
	POIsProcessed = 0;
	CitiesPointsProcessed = 0;
	PolylinesProcessed = 0;
	LinesCreated = 0;
	RegionsProcessed = 0;
	PolygonsCreated = 0;
	
	StartDate = new Date();
	
	if (Options.NamesFileName != undefined) {
		ImportNames(Options)
	}

	if (Options.CutFileName != undefined) {
		Options.CutRegionObject = new Array();
		ProcessFile(Options.CutFileName, Options.CutRegionObject);
	}
	
	SectionsProcessed = 0;
	NodesProcessed = 0;
	POIsProcessed = 0;
	CitiesPointsProcessed = 0;
	PolylinesProcessed = 0;
	LinesCreated = 0;
	RegionsProcessed = 0;
	PolygonsCreated = 0;
	
	ProcessFile(SourceFileName, DestinationFileOrFolderName, Options);
	
	if (DestinationFile != undefined)
		if (Options == undefined || Options.SplitMode == 0)
			DestinationFile.Close();
		else {
			
			DestinationFile.Close();
			while (BlockFiles.length > 0) {
				BlockFile = BlockFiles.shift();
				BlockFile.File.Close();
			}
		}
	
	StatusMessage("");
	StatusMessage("");

	EndDate = new Date();
	TimeElapsed = new Date(EndDate.valueOf() - StartDate.valueOf());
	
//	WScript.Echo("" + SectionsProcessed + "\tsections processed\n"
//	+ POIsProcessed + "\tPOIs processed\n"
//	+ CitiesPointsProcessed + "\tcities/points processed\n"
//	+ PolylinesProcessed + "\tpolylines converted to " + LinesCreated + " lines\n"
//	+ RegionsProcessed + "\tregions converted to " + PolygonsCreated + " polygons\n"
//	+ NodesProcessed + "\tnodes processed\n"
//	+ TimeElapsed.getUTCHours() + ":" + FillLeft(TimeElapsed.getUTCMinutes(), 2, "0") + ":" + FillLeft(TimeElapsed.getUTCSeconds(), 2, "0") + "\telapsed\n"
//	+ (SectionsProcessed * 60000 / TimeElapsed.valueOf()).toFixed(0) + "\tsections per minute\n"
//	+ (NodesProcessed * 1000 / TimeElapsed.valueOf()).toFixed(0) + "\tnodes per second");
}
//-------------------------------------------
function CommandLineMode() {
	return fso.GetBaseName(WScript.FullName).toUpperCase() == "CSCRIPT";
}
//-------------------------------------------
var SourceFileName = "", DestinationFileOrFolderName = "", Options;
var ArgumentNum, Argument, CommandLineSyntaxIsCorrect = true, EqualSighIndex, SwitchName, SwitchValue;
var Blocks, BlockNum, Sticks, StickNum;

if (5.5 > WScript.Version) {
	WScript.Echo(sSplash + "\nError: Windows Script Host 5.5 required.\nDownload and install latest Windows Script Host version from www.microsoft.com.");
} else {

	Options = new Object;
	Options.SplitMode = 0;
	Options.BlocksToSkip = new Array();
	Options.BlocksToStick = new Array();
	Options.NamesFileName = undefined;
	Options.Names = new Array();
	Options.CutFileName = undefined;
	Options.CutRegionObject = undefined;
	Options.SplitPolylines = false;
	Options.SplitRegions = false;
	Options.MergeHoles = true;

	for (ArgumentNum = 0; ArgumentNum < WScript.Arguments.Count(); ArgumentNum++) {
		Argument = WScript.Arguments(ArgumentNum);
		if (Argument.substr(0, 1) != "-" && Argument.substr(0, 1) != "/")
			if (SourceFileName == "")
				SourceFileName = Argument;
			else if (DestinationFileOrFolderName == "")
				DestinationFileOrFolderName = Argument;
			else
				CommandLineSyntaxIsCorrect = false;
		else {
			EqualSighIndex = Argument.indexOf("=");
			SwitchName = (EqualSighIndex == -1 ? Argument.substring(1, Argument.length) : Argument.substring(1, EqualSighIndex)).toLowerCase();
			SwitchValue = EqualSighIndex == -1 ? true : Argument.substring(EqualSighIndex + 1, Argument.length);
			if (SwitchName == "split") {
				if (SwitchValue == "0")
					Options.SplitMode = 0; // Do not split map
				else if (SwitchValue == "1")
					Options.SplitMode = 1; // Split map: 2 by longitude, 2 by latitude
				else if (SwitchValue == "2")
					Options.SplitMode = 2; // Split map: 6 by longitude, 4 by latitude, doubled and quadded near poles (like 1:1 000 000 map)
				else if (SwitchValue == "3")
					Options.SplitMode = 3; // Split map: 3 by longitude, 2 by latitude, doubled and quadded near poles (like 1:500 000 map)
				else if (SwitchValue == "4")
					Options.SplitMode = 4; // Split map: 1 by longitude, 040' by latitude, doubled and quadded near poles (like 1:200 000 map)
				else if (SwitchValue == "5")
					Options.SplitMode = 5; // Split map: 030' by longitude, 020' by latitude, doubled and quadded near poles (like 1:100 000 map)
				else
					CommandLineSyntaxIsCorrect = false;
			} else if (SwitchName == "skipblocks") {
				Blocks = SwitchValue.toLowerCase().split(",");
				for (BlockNum = 0; BlockNum < Blocks.length; BlockNum++) {
					if ("0123456789".indexOf(Blocks[BlockNum].substr(0, 1)) != -1) // Decimal block ID
						Options.BlocksToSkip[String(Number(Blocks[BlockNum]))] = true;
					else if (Blocks[BlockNum].substr(0, 1) == "i"
					&& "0123456789".indexOf(Blocks[BlockNum].substr(1, 1)) != -1) // Hexaecimal block ID
						Options.BlocksToSkip[String(eval("0x" + Blocks[BlockNum].substr(1)))] = true;
					else // Block name
						Options.BlocksToSkip[Blocks[BlockNum]] = true;
				}
				//} else if (SwitchName == "stickblocks") {
				//	Sticks = SwitchValue.toLowerCase().split(";");
				//	for (StickNum = 0; StickNum < Sticks.length; StickNum++) {
				//		Blocks = Sticks[StickNum].split(",");
				//		if ("0123456789".indexOf(Blocks[0].substr(0, 1)) != -1) // Decimal block ID
				//			TargetBlock = Number(Blocks[BlockNum]);
				//		else if (Blocks[0].substr(0, 1) == "i"
				//		&& "0123456789".indexOf(Blocks[0].substr(1, 1)) != -1) // Hexaecimal block ID
				//			TargetBlock = eval("0x" + Blocks[0].substr(1));
				//		else // Block name
				//			TargetBlock = Blocks[0];
				//		for (BlockNum = 1; BlockNum < Blocks.length; BlockNum++) {
				//			if ("0123456789".indexOf(Blocks[BlockNum].substr(0, 1)) != -1) // Decimal block ID
				//				Options.BlocksToStick[String(Number(Blocks[BlockNum]))] = TargetBlock;
				//			else if (Blocks[BlockNum].substr(0, 1) == "i"
				//			&& "0123456789".indexOf(Blocks[BlockNum].substr(1, 1)) != -1) // Hexaecimal block ID
				//				Options.BlocksToStick[String(eval("0x" + Blocks[BlockNum].substr(1)))] = TargetBlock;
				//			else // Block name
				//				Options.BlocksToStick[Blocks[BlockNum]] = TargetBlock;
				//		}
				//	}
			} else if (SwitchName == "names") {
				Options.NamesFileName = SwitchValue;
				//} else if (SwitchName == "cut") {
				//	Options.CutFileName = SwitchValue;
			} else if (SwitchName == "splitpolylines") {
				switch (SwitchValue.toLowerCase()) {
					case "n":	Options.SplitPolylines = false;	break;
					case "y":	Options.SplitPolylines = true;	break;
					default:	CommandLineSyntaxIsCorrect = false;
				}
			} else if (SwitchName == "splitregions") {
				switch (SwitchValue.toLowerCase()) {
					case "n":	Options.SplitRegions = false;	break;
					case "y":	Options.SplitRegions = true;	break;
					default:	CommandLineSyntaxIsCorrect = false;
				}
			} else if (SwitchName == "mergeholes") {
				switch (SwitchValue.toLowerCase()) {
					case "n":	Options.MergeHoles = false;	break;
					case "y":	Options.MergeHoles = true;	break;
					default:	CommandLineSyntaxIsCorrect = false;
				}
			} else
				CommandLineSyntaxIsCorrect = false;
		}
	}
	
	if (SourceFileName == "" || (DestinationFileOrFolderName == "" && Options.SplitMode == 0)
	|| Options.SplitMode == 0 && (Options.BlocksToSkip.length > 0 || Options.BlocksToStick.length > 0))
		CommandLineSyntaxIsCorrect = false;

	if (!CommandLineSyntaxIsCorrect)
		WScript.Echo(sHelp);
	else {
		if (CommandLineMode())
			WScript.StdOut.WriteLine(sSplash);
		// Process arguments
		ProcessFiles(SourceFileName, DestinationFileOrFolderName, Options);
	}
}
